// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at https://mozilla.org/MPL/2.0/.


use error::HpackDecodeError;


pub fn decompress_literal(
        literal: &[u8],
        buffer: &mut Vec<u8>
    ) -> Result<usize, HpackDecodeError> {
    let mut written = 0;
    let mut state = 0;
    let mut accept = true;
    let mut t: (u8, u8, u8);
    for byte in literal.iter() {
        t = DECODE_TABLE[state][(byte >> 4) as usize];
        if t.1 & 4 == 4 {
            return Err(HpackDecodeError::InvalidCompressedLiteral);
        } else if t.1 & 2 == 2 {
            buffer.push(t.2);
            written += 1;
        }
        t = DECODE_TABLE[t.0 as usize][(byte & 0xf) as usize];
        if t.1 & 4 == 4 {
            return Err(HpackDecodeError::InvalidCompressedLiteral);
        } else if t.1 & 2 == 2 {
            buffer.push(t.2);
            written += 1;
        }
        state = t.0 as usize;
        accept = t.1 & 1 == 1;
    }
    if !accept {
        return Err(HpackDecodeError::InvalidCompressedLiteral);
    }
    Ok(written)
}


#[cfg(test)]
mod test {
    extern crate test;
    use self::test::Bencher;
    use super::decompress_literal;


    #[test]
    fn test_decompress_literal_no_cache() {
        let literal = b"\xa8\xeb\x10d\x9c\xbf";
        let mut buf = Vec::with_capacity(32);
        let read = decompress_literal(literal, &mut buf).unwrap();
        assert_eq!(read, 8);
        assert_eq!(
            &buf[..],
            b"no-cache"
        );
    }


    #[test]
    fn test_decompress_literal_datetime() {
        let literal = b"\xd0z\xbe\x94\x10T\xd4D\xa8 \x05\x95\x04\x0b\x81f\xe0\x82\xa6-\x1b\xff";
        let mut buf = Vec::with_capacity(32);
        let read = decompress_literal(literal, &mut buf).unwrap();
        assert_eq!(read, 29);
        assert_eq!(
            &buf[..],
            b"Mon, 21 Oct 2013 20:13:21 GMT"
        );
    }


    #[test]
    fn test_decompress_literal_token_1() {
        let literal = b"\x0bS\x9f\x01=x\xd3\xa7E\xdc\x9cxnn\xeb\xb0\xdfwm\xfcF\xbf\xdf+Wq\x17W\x19\xb2\xc7v\xff\xb9A\x07";
        let mut buf = Vec::with_capacity(50);
        let read = decompress_literal(literal, &mut buf).unwrap();
        assert_eq!(read, 44);
        assert_eq!(
            buf,
            b"14hoEcywNNMBIVUS5B7AD7RDGiDvJ4BGeOVgJbBDzf0=".to_vec()
        );
    }


    #[test]
    fn test_decompress_literal_token_2() {
        let literal = [11, 83, 159, 1, 61, 120, 211, 167, 69, 220, 156, 120, 110, 110, 235, 176, 223, 119, 109, 252, 70, 191, 223, 43, 87, 113, 23, 87, 25, 178, 199, 118, 255, 185, 65, 7].to_vec();
        let mut buf = Vec::with_capacity(50);
        let read = decompress_literal(&literal[..], &mut buf).unwrap();
        assert_eq!(read, 44);
        assert_eq!(
            buf,
            b"14hoEcywNNMBIVUS5B7AD7RDGiDvJ4BGeOVgJbBDzf0=".to_vec()
        );
    }


    #[test]
    fn test_decompress_literal_invalid_first() {
        let literal = b"\xa8\xeb\x10d\x9c\xbf";
        let mut buf = Vec::with_capacity(32);
        let read = decompress_literal(literal, &mut buf).unwrap();
        assert_eq!(read, 8);
        assert_eq!(
            &buf[..],
            b"no-cache"
        );
    }


    #[test]
    fn test_decompress_literal_invalid_second() {
        let literal = b"\xa8\xeb\x10d\x9c\xbf";
        let mut buf = Vec::with_capacity(32);
        let read = decompress_literal(literal, &mut buf).unwrap();
        assert_eq!(read, 8);
        assert_eq!(
            &buf[..],
            b"no-cache"
        );
    }


    #[test]
    fn test_decompress_literal_invalid_end() {
        let literal = b"\xa8\xeb\x10d\x9c\xbf";
        let mut buf = Vec::with_capacity(32);
        let read = decompress_literal(literal, &mut buf).unwrap();
        assert_eq!(read, 8);
        assert_eq!(
            &buf[..],
            b"no-cache"
        );
    }




    #[bench]
    fn bench_decompress_literal_no_cache(b: &mut Bencher) {
        let literal = b"\xa8\xeb\x10d\x9c\xbf";
        let mut buf = Vec::with_capacity(1024);
        b.bytes = literal.len() as u64;
        b.iter(
            || decompress_literal(literal, &mut buf)
        )
    }


    #[bench]
    fn bench_decompress_literal_datetime(b: &mut Bencher) {
        let literal = b"\xd0z\xbe\x94\x10T\xd4D\xa8 \x05\x95\x04\x0b\x81f\xe0\x82\xa6-\x1b\xff";
        let mut buf = Vec::with_capacity(1024);
        b.bytes = literal.len() as u64;
        b.iter(
            || decompress_literal(literal, &mut buf)
        )
    }

}


const DECODE_TABLE: [[(u8, u8, u8); 16]; 256] = [
    [
     (4, 0, 0),
     (5, 0, 0),
     (7, 0, 0),
     (8, 0, 0),
     (11, 0, 0),
     (12, 0, 0),
     (16, 0, 0),
     (19, 0, 0),
     (25, 0, 0),
     (28, 0, 0),
     (32, 0, 0),
     (35, 0, 0),
     (42, 0, 0),
     (49, 0, 0),
     (57, 0, 0),
     (64, 1, 0),
    ],
    [
     (0, 3, 48),
     (0, 3, 49),
     (0, 3, 50),
     (0, 3, 97),
     (0, 3, 99),
     (0, 3, 101),
     (0, 3, 105),
     (0, 3, 111),
     (0, 3, 115),
     (0, 3, 116),
     (13, 0, 0),
     (14, 0, 0),
     (17, 0, 0),
     (18, 0, 0),
     (20, 0, 0),
     (21, 0, 0),
    ],
    [
     (1, 2, 48),
     (22, 3, 48),
     (1, 2, 49),
     (22, 3, 49),
     (1, 2, 50),
     (22, 3, 50),
     (1, 2, 97),
     (22, 3, 97),
     (1, 2, 99),
     (22, 3, 99),
     (1, 2, 101),
     (22, 3, 101),
     (1, 2, 105),
     (22, 3, 105),
     (1, 2, 111),
     (22, 3, 111),
    ],
    [
     (2, 2, 48),
     (9, 2, 48),
     (23, 2, 48),
     (40, 3, 48),
     (2, 2, 49),
     (9, 2, 49),
     (23, 2, 49),
     (40, 3, 49),
     (2, 2, 50),
     (9, 2, 50),
     (23, 2, 50),
     (40, 3, 50),
     (2, 2, 97),
     (9, 2, 97),
     (23, 2, 97),
     (40, 3, 97),
    ],
    [
     (3, 2, 48),
     (6, 2, 48),
     (10, 2, 48),
     (15, 2, 48),
     (24, 2, 48),
     (31, 2, 48),
     (41, 2, 48),
     (56, 3, 48),
     (3, 2, 49),
     (6, 2, 49),
     (10, 2, 49),
     (15, 2, 49),
     (24, 2, 49),
     (31, 2, 49),
     (41, 2, 49),
     (56, 3, 49),
    ],
    [
     (3, 2, 50),
     (6, 2, 50),
     (10, 2, 50),
     (15, 2, 50),
     (24, 2, 50),
     (31, 2, 50),
     (41, 2, 50),
     (56, 3, 50),
     (3, 2, 97),
     (6, 2, 97),
     (10, 2, 97),
     (15, 2, 97),
     (24, 2, 97),
     (31, 2, 97),
     (41, 2, 97),
     (56, 3, 97),
    ],
    [
     (2, 2, 99),
     (9, 2, 99),
     (23, 2, 99),
     (40, 3, 99),
     (2, 2, 101),
     (9, 2, 101),
     (23, 2, 101),
     (40, 3, 101),
     (2, 2, 105),
     (9, 2, 105),
     (23, 2, 105),
     (40, 3, 105),
     (2, 2, 111),
     (9, 2, 111),
     (23, 2, 111),
     (40, 3, 111),
    ],
    [
     (3, 2, 99),
     (6, 2, 99),
     (10, 2, 99),
     (15, 2, 99),
     (24, 2, 99),
     (31, 2, 99),
     (41, 2, 99),
     (56, 3, 99),
     (3, 2, 101),
     (6, 2, 101),
     (10, 2, 101),
     (15, 2, 101),
     (24, 2, 101),
     (31, 2, 101),
     (41, 2, 101),
     (56, 3, 101),
    ],
    [
     (3, 2, 105),
     (6, 2, 105),
     (10, 2, 105),
     (15, 2, 105),
     (24, 2, 105),
     (31, 2, 105),
     (41, 2, 105),
     (56, 3, 105),
     (3, 2, 111),
     (6, 2, 111),
     (10, 2, 111),
     (15, 2, 111),
     (24, 2, 111),
     (31, 2, 111),
     (41, 2, 111),
     (56, 3, 111),
    ],
    [
     (1, 2, 115),
     (22, 3, 115),
     (1, 2, 116),
     (22, 3, 116),
     (0, 3, 32),
     (0, 3, 37),
     (0, 3, 45),
     (0, 3, 46),
     (0, 3, 47),
     (0, 3, 51),
     (0, 3, 52),
     (0, 3, 53),
     (0, 3, 54),
     (0, 3, 55),
     (0, 3, 56),
     (0, 3, 57),
    ],
    [
     (2, 2, 115),
     (9, 2, 115),
     (23, 2, 115),
     (40, 3, 115),
     (2, 2, 116),
     (9, 2, 116),
     (23, 2, 116),
     (40, 3, 116),
     (1, 2, 32),
     (22, 3, 32),
     (1, 2, 37),
     (22, 3, 37),
     (1, 2, 45),
     (22, 3, 45),
     (1, 2, 46),
     (22, 3, 46),
    ],
    [
     (3, 2, 115),
     (6, 2, 115),
     (10, 2, 115),
     (15, 2, 115),
     (24, 2, 115),
     (31, 2, 115),
     (41, 2, 115),
     (56, 3, 115),
     (3, 2, 116),
     (6, 2, 116),
     (10, 2, 116),
     (15, 2, 116),
     (24, 2, 116),
     (31, 2, 116),
     (41, 2, 116),
     (56, 3, 116),
    ],
    [
     (2, 2, 32),
     (9, 2, 32),
     (23, 2, 32),
     (40, 3, 32),
     (2, 2, 37),
     (9, 2, 37),
     (23, 2, 37),
     (40, 3, 37),
     (2, 2, 45),
     (9, 2, 45),
     (23, 2, 45),
     (40, 3, 45),
     (2, 2, 46),
     (9, 2, 46),
     (23, 2, 46),
     (40, 3, 46),
    ],
    [
     (3, 2, 32),
     (6, 2, 32),
     (10, 2, 32),
     (15, 2, 32),
     (24, 2, 32),
     (31, 2, 32),
     (41, 2, 32),
     (56, 3, 32),
     (3, 2, 37),
     (6, 2, 37),
     (10, 2, 37),
     (15, 2, 37),
     (24, 2, 37),
     (31, 2, 37),
     (41, 2, 37),
     (56, 3, 37),
    ],
    [
     (3, 2, 45),
     (6, 2, 45),
     (10, 2, 45),
     (15, 2, 45),
     (24, 2, 45),
     (31, 2, 45),
     (41, 2, 45),
     (56, 3, 45),
     (3, 2, 46),
     (6, 2, 46),
     (10, 2, 46),
     (15, 2, 46),
     (24, 2, 46),
     (31, 2, 46),
     (41, 2, 46),
     (56, 3, 46),
    ],
    [
     (1, 2, 47),
     (22, 3, 47),
     (1, 2, 51),
     (22, 3, 51),
     (1, 2, 52),
     (22, 3, 52),
     (1, 2, 53),
     (22, 3, 53),
     (1, 2, 54),
     (22, 3, 54),
     (1, 2, 55),
     (22, 3, 55),
     (1, 2, 56),
     (22, 3, 56),
     (1, 2, 57),
     (22, 3, 57),
    ],
    [
     (2, 2, 47),
     (9, 2, 47),
     (23, 2, 47),
     (40, 3, 47),
     (2, 2, 51),
     (9, 2, 51),
     (23, 2, 51),
     (40, 3, 51),
     (2, 2, 52),
     (9, 2, 52),
     (23, 2, 52),
     (40, 3, 52),
     (2, 2, 53),
     (9, 2, 53),
     (23, 2, 53),
     (40, 3, 53),
    ],
    [
     (3, 2, 47),
     (6, 2, 47),
     (10, 2, 47),
     (15, 2, 47),
     (24, 2, 47),
     (31, 2, 47),
     (41, 2, 47),
     (56, 3, 47),
     (3, 2, 51),
     (6, 2, 51),
     (10, 2, 51),
     (15, 2, 51),
     (24, 2, 51),
     (31, 2, 51),
     (41, 2, 51),
     (56, 3, 51),
    ],
    [
     (3, 2, 52),
     (6, 2, 52),
     (10, 2, 52),
     (15, 2, 52),
     (24, 2, 52),
     (31, 2, 52),
     (41, 2, 52),
     (56, 3, 52),
     (3, 2, 53),
     (6, 2, 53),
     (10, 2, 53),
     (15, 2, 53),
     (24, 2, 53),
     (31, 2, 53),
     (41, 2, 53),
     (56, 3, 53),
    ],
    [
     (2, 2, 54),
     (9, 2, 54),
     (23, 2, 54),
     (40, 3, 54),
     (2, 2, 55),
     (9, 2, 55),
     (23, 2, 55),
     (40, 3, 55),
     (2, 2, 56),
     (9, 2, 56),
     (23, 2, 56),
     (40, 3, 56),
     (2, 2, 57),
     (9, 2, 57),
     (23, 2, 57),
     (40, 3, 57),
    ],
    [
     (3, 2, 54),
     (6, 2, 54),
     (10, 2, 54),
     (15, 2, 54),
     (24, 2, 54),
     (31, 2, 54),
     (41, 2, 54),
     (56, 3, 54),
     (3, 2, 55),
     (6, 2, 55),
     (10, 2, 55),
     (15, 2, 55),
     (24, 2, 55),
     (31, 2, 55),
     (41, 2, 55),
     (56, 3, 55),
    ],
    [
     (3, 2, 56),
     (6, 2, 56),
     (10, 2, 56),
     (15, 2, 56),
     (24, 2, 56),
     (31, 2, 56),
     (41, 2, 56),
     (56, 3, 56),
     (3, 2, 57),
     (6, 2, 57),
     (10, 2, 57),
     (15, 2, 57),
     (24, 2, 57),
     (31, 2, 57),
     (41, 2, 57),
     (56, 3, 57),
    ],
    [
     (26, 0, 0),
     (27, 0, 0),
     (29, 0, 0),
     (30, 0, 0),
     (33, 0, 0),
     (34, 0, 0),
     (36, 0, 0),
     (37, 0, 0),
     (43, 0, 0),
     (46, 0, 0),
     (50, 0, 0),
     (53, 0, 0),
     (58, 0, 0),
     (61, 0, 0),
     (65, 0, 0),
     (68, 1, 0),
    ],
    [
     (0, 3, 61),
     (0, 3, 65),
     (0, 3, 95),
     (0, 3, 98),
     (0, 3, 100),
     (0, 3, 102),
     (0, 3, 103),
     (0, 3, 104),
     (0, 3, 108),
     (0, 3, 109),
     (0, 3, 110),
     (0, 3, 112),
     (0, 3, 114),
     (0, 3, 117),
     (38, 0, 0),
     (39, 0, 0),
    ],
    [
     (1, 2, 61),
     (22, 3, 61),
     (1, 2, 65),
     (22, 3, 65),
     (1, 2, 95),
     (22, 3, 95),
     (1, 2, 98),
     (22, 3, 98),
     (1, 2, 100),
     (22, 3, 100),
     (1, 2, 102),
     (22, 3, 102),
     (1, 2, 103),
     (22, 3, 103),
     (1, 2, 104),
     (22, 3, 104),
    ],
    [
     (2, 2, 61),
     (9, 2, 61),
     (23, 2, 61),
     (40, 3, 61),
     (2, 2, 65),
     (9, 2, 65),
     (23, 2, 65),
     (40, 3, 65),
     (2, 2, 95),
     (9, 2, 95),
     (23, 2, 95),
     (40, 3, 95),
     (2, 2, 98),
     (9, 2, 98),
     (23, 2, 98),
     (40, 3, 98),
    ],
    [
     (3, 2, 61),
     (6, 2, 61),
     (10, 2, 61),
     (15, 2, 61),
     (24, 2, 61),
     (31, 2, 61),
     (41, 2, 61),
     (56, 3, 61),
     (3, 2, 65),
     (6, 2, 65),
     (10, 2, 65),
     (15, 2, 65),
     (24, 2, 65),
     (31, 2, 65),
     (41, 2, 65),
     (56, 3, 65),
    ],
    [
     (3, 2, 95),
     (6, 2, 95),
     (10, 2, 95),
     (15, 2, 95),
     (24, 2, 95),
     (31, 2, 95),
     (41, 2, 95),
     (56, 3, 95),
     (3, 2, 98),
     (6, 2, 98),
     (10, 2, 98),
     (15, 2, 98),
     (24, 2, 98),
     (31, 2, 98),
     (41, 2, 98),
     (56, 3, 98),
    ],
    [
     (2, 2, 100),
     (9, 2, 100),
     (23, 2, 100),
     (40, 3, 100),
     (2, 2, 102),
     (9, 2, 102),
     (23, 2, 102),
     (40, 3, 102),
     (2, 2, 103),
     (9, 2, 103),
     (23, 2, 103),
     (40, 3, 103),
     (2, 2, 104),
     (9, 2, 104),
     (23, 2, 104),
     (40, 3, 104),
    ],
    [
     (3, 2, 100),
     (6, 2, 100),
     (10, 2, 100),
     (15, 2, 100),
     (24, 2, 100),
     (31, 2, 100),
     (41, 2, 100),
     (56, 3, 100),
     (3, 2, 102),
     (6, 2, 102),
     (10, 2, 102),
     (15, 2, 102),
     (24, 2, 102),
     (31, 2, 102),
     (41, 2, 102),
     (56, 3, 102),
    ],
    [
     (3, 2, 103),
     (6, 2, 103),
     (10, 2, 103),
     (15, 2, 103),
     (24, 2, 103),
     (31, 2, 103),
     (41, 2, 103),
     (56, 3, 103),
     (3, 2, 104),
     (6, 2, 104),
     (10, 2, 104),
     (15, 2, 104),
     (24, 2, 104),
     (31, 2, 104),
     (41, 2, 104),
     (56, 3, 104),
    ],
    [
     (1, 2, 108),
     (22, 3, 108),
     (1, 2, 109),
     (22, 3, 109),
     (1, 2, 110),
     (22, 3, 110),
     (1, 2, 112),
     (22, 3, 112),
     (1, 2, 114),
     (22, 3, 114),
     (1, 2, 117),
     (22, 3, 117),
     (0, 3, 58),
     (0, 3, 66),
     (0, 3, 67),
     (0, 3, 68),
    ],
    [
     (2, 2, 108),
     (9, 2, 108),
     (23, 2, 108),
     (40, 3, 108),
     (2, 2, 109),
     (9, 2, 109),
     (23, 2, 109),
     (40, 3, 109),
     (2, 2, 110),
     (9, 2, 110),
     (23, 2, 110),
     (40, 3, 110),
     (2, 2, 112),
     (9, 2, 112),
     (23, 2, 112),
     (40, 3, 112),
    ],
    [
     (3, 2, 108),
     (6, 2, 108),
     (10, 2, 108),
     (15, 2, 108),
     (24, 2, 108),
     (31, 2, 108),
     (41, 2, 108),
     (56, 3, 108),
     (3, 2, 109),
     (6, 2, 109),
     (10, 2, 109),
     (15, 2, 109),
     (24, 2, 109),
     (31, 2, 109),
     (41, 2, 109),
     (56, 3, 109),
    ],
    [
     (3, 2, 110),
     (6, 2, 110),
     (10, 2, 110),
     (15, 2, 110),
     (24, 2, 110),
     (31, 2, 110),
     (41, 2, 110),
     (56, 3, 110),
     (3, 2, 112),
     (6, 2, 112),
     (10, 2, 112),
     (15, 2, 112),
     (24, 2, 112),
     (31, 2, 112),
     (41, 2, 112),
     (56, 3, 112),
    ],
    [
     (2, 2, 114),
     (9, 2, 114),
     (23, 2, 114),
     (40, 3, 114),
     (2, 2, 117),
     (9, 2, 117),
     (23, 2, 117),
     (40, 3, 117),
     (1, 2, 58),
     (22, 3, 58),
     (1, 2, 66),
     (22, 3, 66),
     (1, 2, 67),
     (22, 3, 67),
     (1, 2, 68),
     (22, 3, 68),
    ],
    [
     (3, 2, 114),
     (6, 2, 114),
     (10, 2, 114),
     (15, 2, 114),
     (24, 2, 114),
     (31, 2, 114),
     (41, 2, 114),
     (56, 3, 114),
     (3, 2, 117),
     (6, 2, 117),
     (10, 2, 117),
     (15, 2, 117),
     (24, 2, 117),
     (31, 2, 117),
     (41, 2, 117),
     (56, 3, 117),
    ],
    [
     (2, 2, 58),
     (9, 2, 58),
     (23, 2, 58),
     (40, 3, 58),
     (2, 2, 66),
     (9, 2, 66),
     (23, 2, 66),
     (40, 3, 66),
     (2, 2, 67),
     (9, 2, 67),
     (23, 2, 67),
     (40, 3, 67),
     (2, 2, 68),
     (9, 2, 68),
     (23, 2, 68),
     (40, 3, 68),
    ],
    [
     (3, 2, 58),
     (6, 2, 58),
     (10, 2, 58),
     (15, 2, 58),
     (24, 2, 58),
     (31, 2, 58),
     (41, 2, 58),
     (56, 3, 58),
     (3, 2, 66),
     (6, 2, 66),
     (10, 2, 66),
     (15, 2, 66),
     (24, 2, 66),
     (31, 2, 66),
     (41, 2, 66),
     (56, 3, 66),
    ],
    [
     (3, 2, 67),
     (6, 2, 67),
     (10, 2, 67),
     (15, 2, 67),
     (24, 2, 67),
     (31, 2, 67),
     (41, 2, 67),
     (56, 3, 67),
     (3, 2, 68),
     (6, 2, 68),
     (10, 2, 68),
     (15, 2, 68),
     (24, 2, 68),
     (31, 2, 68),
     (41, 2, 68),
     (56, 3, 68),
    ],
    [
     (44, 0, 0),
     (45, 0, 0),
     (47, 0, 0),
     (48, 0, 0),
     (51, 0, 0),
     (52, 0, 0),
     (54, 0, 0),
     (55, 0, 0),
     (59, 0, 0),
     (60, 0, 0),
     (62, 0, 0),
     (63, 0, 0),
     (66, 0, 0),
     (67, 0, 0),
     (69, 0, 0),
     (72, 1, 0),
    ],
    [
     (0, 3, 69),
     (0, 3, 70),
     (0, 3, 71),
     (0, 3, 72),
     (0, 3, 73),
     (0, 3, 74),
     (0, 3, 75),
     (0, 3, 76),
     (0, 3, 77),
     (0, 3, 78),
     (0, 3, 79),
     (0, 3, 80),
     (0, 3, 81),
     (0, 3, 82),
     (0, 3, 83),
     (0, 3, 84),
    ],
    [
     (1, 2, 69),
     (22, 3, 69),
     (1, 2, 70),
     (22, 3, 70),
     (1, 2, 71),
     (22, 3, 71),
     (1, 2, 72),
     (22, 3, 72),
     (1, 2, 73),
     (22, 3, 73),
     (1, 2, 74),
     (22, 3, 74),
     (1, 2, 75),
     (22, 3, 75),
     (1, 2, 76),
     (22, 3, 76),
    ],
    [
     (2, 2, 69),
     (9, 2, 69),
     (23, 2, 69),
     (40, 3, 69),
     (2, 2, 70),
     (9, 2, 70),
     (23, 2, 70),
     (40, 3, 70),
     (2, 2, 71),
     (9, 2, 71),
     (23, 2, 71),
     (40, 3, 71),
     (2, 2, 72),
     (9, 2, 72),
     (23, 2, 72),
     (40, 3, 72),
    ],
    [
     (3, 2, 69),
     (6, 2, 69),
     (10, 2, 69),
     (15, 2, 69),
     (24, 2, 69),
     (31, 2, 69),
     (41, 2, 69),
     (56, 3, 69),
     (3, 2, 70),
     (6, 2, 70),
     (10, 2, 70),
     (15, 2, 70),
     (24, 2, 70),
     (31, 2, 70),
     (41, 2, 70),
     (56, 3, 70),
    ],
    [
     (3, 2, 71),
     (6, 2, 71),
     (10, 2, 71),
     (15, 2, 71),
     (24, 2, 71),
     (31, 2, 71),
     (41, 2, 71),
     (56, 3, 71),
     (3, 2, 72),
     (6, 2, 72),
     (10, 2, 72),
     (15, 2, 72),
     (24, 2, 72),
     (31, 2, 72),
     (41, 2, 72),
     (56, 3, 72),
    ],
    [
     (2, 2, 73),
     (9, 2, 73),
     (23, 2, 73),
     (40, 3, 73),
     (2, 2, 74),
     (9, 2, 74),
     (23, 2, 74),
     (40, 3, 74),
     (2, 2, 75),
     (9, 2, 75),
     (23, 2, 75),
     (40, 3, 75),
     (2, 2, 76),
     (9, 2, 76),
     (23, 2, 76),
     (40, 3, 76),
    ],
    [
     (3, 2, 73),
     (6, 2, 73),
     (10, 2, 73),
     (15, 2, 73),
     (24, 2, 73),
     (31, 2, 73),
     (41, 2, 73),
     (56, 3, 73),
     (3, 2, 74),
     (6, 2, 74),
     (10, 2, 74),
     (15, 2, 74),
     (24, 2, 74),
     (31, 2, 74),
     (41, 2, 74),
     (56, 3, 74),
    ],
    [
     (3, 2, 75),
     (6, 2, 75),
     (10, 2, 75),
     (15, 2, 75),
     (24, 2, 75),
     (31, 2, 75),
     (41, 2, 75),
     (56, 3, 75),
     (3, 2, 76),
     (6, 2, 76),
     (10, 2, 76),
     (15, 2, 76),
     (24, 2, 76),
     (31, 2, 76),
     (41, 2, 76),
     (56, 3, 76),
    ],
    [
     (1, 2, 77),
     (22, 3, 77),
     (1, 2, 78),
     (22, 3, 78),
     (1, 2, 79),
     (22, 3, 79),
     (1, 2, 80),
     (22, 3, 80),
     (1, 2, 81),
     (22, 3, 81),
     (1, 2, 82),
     (22, 3, 82),
     (1, 2, 83),
     (22, 3, 83),
     (1, 2, 84),
     (22, 3, 84),
    ],
    [
     (2, 2, 77),
     (9, 2, 77),
     (23, 2, 77),
     (40, 3, 77),
     (2, 2, 78),
     (9, 2, 78),
     (23, 2, 78),
     (40, 3, 78),
     (2, 2, 79),
     (9, 2, 79),
     (23, 2, 79),
     (40, 3, 79),
     (2, 2, 80),
     (9, 2, 80),
     (23, 2, 80),
     (40, 3, 80),
    ],
    [
     (3, 2, 77),
     (6, 2, 77),
     (10, 2, 77),
     (15, 2, 77),
     (24, 2, 77),
     (31, 2, 77),
     (41, 2, 77),
     (56, 3, 77),
     (3, 2, 78),
     (6, 2, 78),
     (10, 2, 78),
     (15, 2, 78),
     (24, 2, 78),
     (31, 2, 78),
     (41, 2, 78),
     (56, 3, 78),
    ],
    [
     (3, 2, 79),
     (6, 2, 79),
     (10, 2, 79),
     (15, 2, 79),
     (24, 2, 79),
     (31, 2, 79),
     (41, 2, 79),
     (56, 3, 79),
     (3, 2, 80),
     (6, 2, 80),
     (10, 2, 80),
     (15, 2, 80),
     (24, 2, 80),
     (31, 2, 80),
     (41, 2, 80),
     (56, 3, 80),
    ],
    [
     (2, 2, 81),
     (9, 2, 81),
     (23, 2, 81),
     (40, 3, 81),
     (2, 2, 82),
     (9, 2, 82),
     (23, 2, 82),
     (40, 3, 82),
     (2, 2, 83),
     (9, 2, 83),
     (23, 2, 83),
     (40, 3, 83),
     (2, 2, 84),
     (9, 2, 84),
     (23, 2, 84),
     (40, 3, 84),
    ],
    [
     (3, 2, 81),
     (6, 2, 81),
     (10, 2, 81),
     (15, 2, 81),
     (24, 2, 81),
     (31, 2, 81),
     (41, 2, 81),
     (56, 3, 81),
     (3, 2, 82),
     (6, 2, 82),
     (10, 2, 82),
     (15, 2, 82),
     (24, 2, 82),
     (31, 2, 82),
     (41, 2, 82),
     (56, 3, 82),
    ],
    [
     (3, 2, 83),
     (6, 2, 83),
     (10, 2, 83),
     (15, 2, 83),
     (24, 2, 83),
     (31, 2, 83),
     (41, 2, 83),
     (56, 3, 83),
     (3, 2, 84),
     (6, 2, 84),
     (10, 2, 84),
     (15, 2, 84),
     (24, 2, 84),
     (31, 2, 84),
     (41, 2, 84),
     (56, 3, 84),
    ],
    [
     (0, 3, 85),
     (0, 3, 86),
     (0, 3, 87),
     (0, 3, 89),
     (0, 3, 106),
     (0, 3, 107),
     (0, 3, 113),
     (0, 3, 118),
     (0, 3, 119),
     (0, 3, 120),
     (0, 3, 121),
     (0, 3, 122),
     (70, 0, 0),
     (71, 0, 0),
     (73, 0, 0),
     (74, 1, 0),
    ],
    [
     (1, 2, 85),
     (22, 3, 85),
     (1, 2, 86),
     (22, 3, 86),
     (1, 2, 87),
     (22, 3, 87),
     (1, 2, 89),
     (22, 3, 89),
     (1, 2, 106),
     (22, 3, 106),
     (1, 2, 107),
     (22, 3, 107),
     (1, 2, 113),
     (22, 3, 113),
     (1, 2, 118),
     (22, 3, 118),
    ],
    [
     (2, 2, 85),
     (9, 2, 85),
     (23, 2, 85),
     (40, 3, 85),
     (2, 2, 86),
     (9, 2, 86),
     (23, 2, 86),
     (40, 3, 86),
     (2, 2, 87),
     (9, 2, 87),
     (23, 2, 87),
     (40, 3, 87),
     (2, 2, 89),
     (9, 2, 89),
     (23, 2, 89),
     (40, 3, 89),
    ],
    [
     (3, 2, 85),
     (6, 2, 85),
     (10, 2, 85),
     (15, 2, 85),
     (24, 2, 85),
     (31, 2, 85),
     (41, 2, 85),
     (56, 3, 85),
     (3, 2, 86),
     (6, 2, 86),
     (10, 2, 86),
     (15, 2, 86),
     (24, 2, 86),
     (31, 2, 86),
     (41, 2, 86),
     (56, 3, 86),
    ],
    [
     (3, 2, 87),
     (6, 2, 87),
     (10, 2, 87),
     (15, 2, 87),
     (24, 2, 87),
     (31, 2, 87),
     (41, 2, 87),
     (56, 3, 87),
     (3, 2, 89),
     (6, 2, 89),
     (10, 2, 89),
     (15, 2, 89),
     (24, 2, 89),
     (31, 2, 89),
     (41, 2, 89),
     (56, 3, 89),
    ],
    [
     (2, 2, 106),
     (9, 2, 106),
     (23, 2, 106),
     (40, 3, 106),
     (2, 2, 107),
     (9, 2, 107),
     (23, 2, 107),
     (40, 3, 107),
     (2, 2, 113),
     (9, 2, 113),
     (23, 2, 113),
     (40, 3, 113),
     (2, 2, 118),
     (9, 2, 118),
     (23, 2, 118),
     (40, 3, 118),
    ],
    [
     (3, 2, 106),
     (6, 2, 106),
     (10, 2, 106),
     (15, 2, 106),
     (24, 2, 106),
     (31, 2, 106),
     (41, 2, 106),
     (56, 3, 106),
     (3, 2, 107),
     (6, 2, 107),
     (10, 2, 107),
     (15, 2, 107),
     (24, 2, 107),
     (31, 2, 107),
     (41, 2, 107),
     (56, 3, 107),
    ],
    [
     (3, 2, 113),
     (6, 2, 113),
     (10, 2, 113),
     (15, 2, 113),
     (24, 2, 113),
     (31, 2, 113),
     (41, 2, 113),
     (56, 3, 113),
     (3, 2, 118),
     (6, 2, 118),
     (10, 2, 118),
     (15, 2, 118),
     (24, 2, 118),
     (31, 2, 118),
     (41, 2, 118),
     (56, 3, 118),
    ],
    [
     (1, 2, 119),
     (22, 3, 119),
     (1, 2, 120),
     (22, 3, 120),
     (1, 2, 121),
     (22, 3, 121),
     (1, 2, 122),
     (22, 3, 122),
     (0, 3, 38),
     (0, 3, 42),
     (0, 3, 44),
     (0, 3, 59),
     (0, 3, 88),
     (0, 3, 90),
     (75, 0, 0),
     (78, 0, 0),
    ],
    [
     (2, 2, 119),
     (9, 2, 119),
     (23, 2, 119),
     (40, 3, 119),
     (2, 2, 120),
     (9, 2, 120),
     (23, 2, 120),
     (40, 3, 120),
     (2, 2, 121),
     (9, 2, 121),
     (23, 2, 121),
     (40, 3, 121),
     (2, 2, 122),
     (9, 2, 122),
     (23, 2, 122),
     (40, 3, 122),
    ],
    [
     (3, 2, 119),
     (6, 2, 119),
     (10, 2, 119),
     (15, 2, 119),
     (24, 2, 119),
     (31, 2, 119),
     (41, 2, 119),
     (56, 3, 119),
     (3, 2, 120),
     (6, 2, 120),
     (10, 2, 120),
     (15, 2, 120),
     (24, 2, 120),
     (31, 2, 120),
     (41, 2, 120),
     (56, 3, 120),
    ],
    [
     (3, 2, 121),
     (6, 2, 121),
     (10, 2, 121),
     (15, 2, 121),
     (24, 2, 121),
     (31, 2, 121),
     (41, 2, 121),
     (56, 3, 121),
     (3, 2, 122),
     (6, 2, 122),
     (10, 2, 122),
     (15, 2, 122),
     (24, 2, 122),
     (31, 2, 122),
     (41, 2, 122),
     (56, 3, 122),
    ],
    [
     (1, 2, 38),
     (22, 3, 38),
     (1, 2, 42),
     (22, 3, 42),
     (1, 2, 44),
     (22, 3, 44),
     (1, 2, 59),
     (22, 3, 59),
     (1, 2, 88),
     (22, 3, 88),
     (1, 2, 90),
     (22, 3, 90),
     (76, 0, 0),
     (77, 0, 0),
     (79, 0, 0),
     (81, 0, 0),
    ],
    [
     (2, 2, 38),
     (9, 2, 38),
     (23, 2, 38),
     (40, 3, 38),
     (2, 2, 42),
     (9, 2, 42),
     (23, 2, 42),
     (40, 3, 42),
     (2, 2, 44),
     (9, 2, 44),
     (23, 2, 44),
     (40, 3, 44),
     (2, 2, 59),
     (9, 2, 59),
     (23, 2, 59),
     (40, 3, 59),
    ],
    [
     (3, 2, 38),
     (6, 2, 38),
     (10, 2, 38),
     (15, 2, 38),
     (24, 2, 38),
     (31, 2, 38),
     (41, 2, 38),
     (56, 3, 38),
     (3, 2, 42),
     (6, 2, 42),
     (10, 2, 42),
     (15, 2, 42),
     (24, 2, 42),
     (31, 2, 42),
     (41, 2, 42),
     (56, 3, 42),
    ],
    [
     (3, 2, 44),
     (6, 2, 44),
     (10, 2, 44),
     (15, 2, 44),
     (24, 2, 44),
     (31, 2, 44),
     (41, 2, 44),
     (56, 3, 44),
     (3, 2, 59),
     (6, 2, 59),
     (10, 2, 59),
     (15, 2, 59),
     (24, 2, 59),
     (31, 2, 59),
     (41, 2, 59),
     (56, 3, 59),
    ],
    [
     (2, 2, 88),
     (9, 2, 88),
     (23, 2, 88),
     (40, 3, 88),
     (2, 2, 90),
     (9, 2, 90),
     (23, 2, 90),
     (40, 3, 90),
     (0, 3, 33),
     (0, 3, 34),
     (0, 3, 40),
     (0, 3, 41),
     (0, 3, 63),
     (80, 0, 0),
     (82, 0, 0),
     (84, 0, 0),
    ],
    [
     (3, 2, 88),
     (6, 2, 88),
     (10, 2, 88),
     (15, 2, 88),
     (24, 2, 88),
     (31, 2, 88),
     (41, 2, 88),
     (56, 3, 88),
     (3, 2, 90),
     (6, 2, 90),
     (10, 2, 90),
     (15, 2, 90),
     (24, 2, 90),
     (31, 2, 90),
     (41, 2, 90),
     (56, 3, 90),
    ],
    [
     (1, 2, 33),
     (22, 3, 33),
     (1, 2, 34),
     (22, 3, 34),
     (1, 2, 40),
     (22, 3, 40),
     (1, 2, 41),
     (22, 3, 41),
     (1, 2, 63),
     (22, 3, 63),
     (0, 3, 39),
     (0, 3, 43),
     (0, 3, 124),
     (83, 0, 0),
     (85, 0, 0),
     (88, 0, 0),
    ],
    [
     (2, 2, 33),
     (9, 2, 33),
     (23, 2, 33),
     (40, 3, 33),
     (2, 2, 34),
     (9, 2, 34),
     (23, 2, 34),
     (40, 3, 34),
     (2, 2, 40),
     (9, 2, 40),
     (23, 2, 40),
     (40, 3, 40),
     (2, 2, 41),
     (9, 2, 41),
     (23, 2, 41),
     (40, 3, 41),
    ],
    [
     (3, 2, 33),
     (6, 2, 33),
     (10, 2, 33),
     (15, 2, 33),
     (24, 2, 33),
     (31, 2, 33),
     (41, 2, 33),
     (56, 3, 33),
     (3, 2, 34),
     (6, 2, 34),
     (10, 2, 34),
     (15, 2, 34),
     (24, 2, 34),
     (31, 2, 34),
     (41, 2, 34),
     (56, 3, 34),
    ],
    [
     (3, 2, 40),
     (6, 2, 40),
     (10, 2, 40),
     (15, 2, 40),
     (24, 2, 40),
     (31, 2, 40),
     (41, 2, 40),
     (56, 3, 40),
     (3, 2, 41),
     (6, 2, 41),
     (10, 2, 41),
     (15, 2, 41),
     (24, 2, 41),
     (31, 2, 41),
     (41, 2, 41),
     (56, 3, 41),
    ],
    [
     (2, 2, 63),
     (9, 2, 63),
     (23, 2, 63),
     (40, 3, 63),
     (1, 2, 39),
     (22, 3, 39),
     (1, 2, 43),
     (22, 3, 43),
     (1, 2, 124),
     (22, 3, 124),
     (0, 3, 35),
     (0, 3, 62),
     (86, 0, 0),
     (87, 0, 0),
     (89, 0, 0),
     (90, 0, 0),
    ],
    [
     (3, 2, 63),
     (6, 2, 63),
     (10, 2, 63),
     (15, 2, 63),
     (24, 2, 63),
     (31, 2, 63),
     (41, 2, 63),
     (56, 3, 63),
     (2, 2, 39),
     (9, 2, 39),
     (23, 2, 39),
     (40, 3, 39),
     (2, 2, 43),
     (9, 2, 43),
     (23, 2, 43),
     (40, 3, 43),
    ],
    [
     (3, 2, 39),
     (6, 2, 39),
     (10, 2, 39),
     (15, 2, 39),
     (24, 2, 39),
     (31, 2, 39),
     (41, 2, 39),
     (56, 3, 39),
     (3, 2, 43),
     (6, 2, 43),
     (10, 2, 43),
     (15, 2, 43),
     (24, 2, 43),
     (31, 2, 43),
     (41, 2, 43),
     (56, 3, 43),
    ],
    [
     (2, 2, 124),
     (9, 2, 124),
     (23, 2, 124),
     (40, 3, 124),
     (1, 2, 35),
     (22, 3, 35),
     (1, 2, 62),
     (22, 3, 62),
     (0, 3, 0),
     (0, 3, 36),
     (0, 3, 64),
     (0, 3, 91),
     (0, 3, 93),
     (0, 3, 126),
     (91, 0, 0),
     (92, 0, 0),
    ],
    [
     (3, 2, 124),
     (6, 2, 124),
     (10, 2, 124),
     (15, 2, 124),
     (24, 2, 124),
     (31, 2, 124),
     (41, 2, 124),
     (56, 3, 124),
     (2, 2, 35),
     (9, 2, 35),
     (23, 2, 35),
     (40, 3, 35),
     (2, 2, 62),
     (9, 2, 62),
     (23, 2, 62),
     (40, 3, 62),
    ],
    [
     (3, 2, 35),
     (6, 2, 35),
     (10, 2, 35),
     (15, 2, 35),
     (24, 2, 35),
     (31, 2, 35),
     (41, 2, 35),
     (56, 3, 35),
     (3, 2, 62),
     (6, 2, 62),
     (10, 2, 62),
     (15, 2, 62),
     (24, 2, 62),
     (31, 2, 62),
     (41, 2, 62),
     (56, 3, 62),
    ],
    [
     (1, 2, 0),
     (22, 3, 0),
     (1, 2, 36),
     (22, 3, 36),
     (1, 2, 64),
     (22, 3, 64),
     (1, 2, 91),
     (22, 3, 91),
     (1, 2, 93),
     (22, 3, 93),
     (1, 2, 126),
     (22, 3, 126),
     (0, 3, 94),
     (0, 3, 125),
     (93, 0, 0),
     (94, 0, 0),
    ],
    [
     (2, 2, 0),
     (9, 2, 0),
     (23, 2, 0),
     (40, 3, 0),
     (2, 2, 36),
     (9, 2, 36),
     (23, 2, 36),
     (40, 3, 36),
     (2, 2, 64),
     (9, 2, 64),
     (23, 2, 64),
     (40, 3, 64),
     (2, 2, 91),
     (9, 2, 91),
     (23, 2, 91),
     (40, 3, 91),
    ],
    [
     (3, 2, 0),
     (6, 2, 0),
     (10, 2, 0),
     (15, 2, 0),
     (24, 2, 0),
     (31, 2, 0),
     (41, 2, 0),
     (56, 3, 0),
     (3, 2, 36),
     (6, 2, 36),
     (10, 2, 36),
     (15, 2, 36),
     (24, 2, 36),
     (31, 2, 36),
     (41, 2, 36),
     (56, 3, 36),
    ],
    [
     (3, 2, 64),
     (6, 2, 64),
     (10, 2, 64),
     (15, 2, 64),
     (24, 2, 64),
     (31, 2, 64),
     (41, 2, 64),
     (56, 3, 64),
     (3, 2, 91),
     (6, 2, 91),
     (10, 2, 91),
     (15, 2, 91),
     (24, 2, 91),
     (31, 2, 91),
     (41, 2, 91),
     (56, 3, 91),
    ],
    [
     (2, 2, 93),
     (9, 2, 93),
     (23, 2, 93),
     (40, 3, 93),
     (2, 2, 126),
     (9, 2, 126),
     (23, 2, 126),
     (40, 3, 126),
     (1, 2, 94),
     (22, 3, 94),
     (1, 2, 125),
     (22, 3, 125),
     (0, 3, 60),
     (0, 3, 96),
     (0, 3, 123),
     (95, 0, 0),
    ],
    [
     (3, 2, 93),
     (6, 2, 93),
     (10, 2, 93),
     (15, 2, 93),
     (24, 2, 93),
     (31, 2, 93),
     (41, 2, 93),
     (56, 3, 93),
     (3, 2, 126),
     (6, 2, 126),
     (10, 2, 126),
     (15, 2, 126),
     (24, 2, 126),
     (31, 2, 126),
     (41, 2, 126),
     (56, 3, 126),
    ],
    [
     (2, 2, 94),
     (9, 2, 94),
     (23, 2, 94),
     (40, 3, 94),
     (2, 2, 125),
     (9, 2, 125),
     (23, 2, 125),
     (40, 3, 125),
     (1, 2, 60),
     (22, 3, 60),
     (1, 2, 96),
     (22, 3, 96),
     (1, 2, 123),
     (22, 3, 123),
     (96, 0, 0),
     (110, 0, 0),
    ],
    [
     (3, 2, 94),
     (6, 2, 94),
     (10, 2, 94),
     (15, 2, 94),
     (24, 2, 94),
     (31, 2, 94),
     (41, 2, 94),
     (56, 3, 94),
     (3, 2, 125),
     (6, 2, 125),
     (10, 2, 125),
     (15, 2, 125),
     (24, 2, 125),
     (31, 2, 125),
     (41, 2, 125),
     (56, 3, 125),
    ],
    [
     (2, 2, 60),
     (9, 2, 60),
     (23, 2, 60),
     (40, 3, 60),
     (2, 2, 96),
     (9, 2, 96),
     (23, 2, 96),
     (40, 3, 96),
     (2, 2, 123),
     (9, 2, 123),
     (23, 2, 123),
     (40, 3, 123),
     (97, 0, 0),
     (101, 0, 0),
     (111, 0, 0),
     (133, 0, 0),
    ],
    [
     (3, 2, 60),
     (6, 2, 60),
     (10, 2, 60),
     (15, 2, 60),
     (24, 2, 60),
     (31, 2, 60),
     (41, 2, 60),
     (56, 3, 60),
     (3, 2, 96),
     (6, 2, 96),
     (10, 2, 96),
     (15, 2, 96),
     (24, 2, 96),
     (31, 2, 96),
     (41, 2, 96),
     (56, 3, 96),
    ],
    [
     (3, 2, 123),
     (6, 2, 123),
     (10, 2, 123),
     (15, 2, 123),
     (24, 2, 123),
     (31, 2, 123),
     (41, 2, 123),
     (56, 3, 123),
     (98, 0, 0),
     (99, 0, 0),
     (102, 0, 0),
     (105, 0, 0),
     (112, 0, 0),
     (119, 0, 0),
     (134, 0, 0),
     (153, 0, 0),
    ],
    [
     (0, 3, 92),
     (0, 3, 195),
     (0, 3, 208),
     (100, 0, 0),
     (103, 0, 0),
     (104, 0, 0),
     (106, 0, 0),
     (107, 0, 0),
     (113, 0, 0),
     (116, 0, 0),
     (120, 0, 0),
     (126, 0, 0),
     (135, 0, 0),
     (142, 0, 0),
     (154, 0, 0),
     (169, 0, 0),
    ],
    [
     (1, 2, 92),
     (22, 3, 92),
     (1, 2, 195),
     (22, 3, 195),
     (1, 2, 208),
     (22, 3, 208),
     (0, 3, 128),
     (0, 3, 130),
     (0, 3, 131),
     (0, 3, 162),
     (0, 3, 184),
     (0, 3, 194),
     (0, 3, 224),
     (0, 3, 226),
     (108, 0, 0),
     (109, 0, 0),
    ],
    [
     (2, 2, 92),
     (9, 2, 92),
     (23, 2, 92),
     (40, 3, 92),
     (2, 2, 195),
     (9, 2, 195),
     (23, 2, 195),
     (40, 3, 195),
     (2, 2, 208),
     (9, 2, 208),
     (23, 2, 208),
     (40, 3, 208),
     (1, 2, 128),
     (22, 3, 128),
     (1, 2, 130),
     (22, 3, 130),
    ],
    [
     (3, 2, 92),
     (6, 2, 92),
     (10, 2, 92),
     (15, 2, 92),
     (24, 2, 92),
     (31, 2, 92),
     (41, 2, 92),
     (56, 3, 92),
     (3, 2, 195),
     (6, 2, 195),
     (10, 2, 195),
     (15, 2, 195),
     (24, 2, 195),
     (31, 2, 195),
     (41, 2, 195),
     (56, 3, 195),
    ],
    [
     (3, 2, 208),
     (6, 2, 208),
     (10, 2, 208),
     (15, 2, 208),
     (24, 2, 208),
     (31, 2, 208),
     (41, 2, 208),
     (56, 3, 208),
     (2, 2, 128),
     (9, 2, 128),
     (23, 2, 128),
     (40, 3, 128),
     (2, 2, 130),
     (9, 2, 130),
     (23, 2, 130),
     (40, 3, 130),
    ],
    [
     (3, 2, 128),
     (6, 2, 128),
     (10, 2, 128),
     (15, 2, 128),
     (24, 2, 128),
     (31, 2, 128),
     (41, 2, 128),
     (56, 3, 128),
     (3, 2, 130),
     (6, 2, 130),
     (10, 2, 130),
     (15, 2, 130),
     (24, 2, 130),
     (31, 2, 130),
     (41, 2, 130),
     (56, 3, 130),
    ],
    [
     (1, 2, 131),
     (22, 3, 131),
     (1, 2, 162),
     (22, 3, 162),
     (1, 2, 184),
     (22, 3, 184),
     (1, 2, 194),
     (22, 3, 194),
     (1, 2, 224),
     (22, 3, 224),
     (1, 2, 226),
     (22, 3, 226),
     (0, 3, 153),
     (0, 3, 161),
     (0, 3, 167),
     (0, 3, 172),
    ],
    [
     (2, 2, 131),
     (9, 2, 131),
     (23, 2, 131),
     (40, 3, 131),
     (2, 2, 162),
     (9, 2, 162),
     (23, 2, 162),
     (40, 3, 162),
     (2, 2, 184),
     (9, 2, 184),
     (23, 2, 184),
     (40, 3, 184),
     (2, 2, 194),
     (9, 2, 194),
     (23, 2, 194),
     (40, 3, 194),
    ],
    [
     (3, 2, 131),
     (6, 2, 131),
     (10, 2, 131),
     (15, 2, 131),
     (24, 2, 131),
     (31, 2, 131),
     (41, 2, 131),
     (56, 3, 131),
     (3, 2, 162),
     (6, 2, 162),
     (10, 2, 162),
     (15, 2, 162),
     (24, 2, 162),
     (31, 2, 162),
     (41, 2, 162),
     (56, 3, 162),
    ],
    [
     (3, 2, 184),
     (6, 2, 184),
     (10, 2, 184),
     (15, 2, 184),
     (24, 2, 184),
     (31, 2, 184),
     (41, 2, 184),
     (56, 3, 184),
     (3, 2, 194),
     (6, 2, 194),
     (10, 2, 194),
     (15, 2, 194),
     (24, 2, 194),
     (31, 2, 194),
     (41, 2, 194),
     (56, 3, 194),
    ],
    [
     (2, 2, 224),
     (9, 2, 224),
     (23, 2, 224),
     (40, 3, 224),
     (2, 2, 226),
     (9, 2, 226),
     (23, 2, 226),
     (40, 3, 226),
     (1, 2, 153),
     (22, 3, 153),
     (1, 2, 161),
     (22, 3, 161),
     (1, 2, 167),
     (22, 3, 167),
     (1, 2, 172),
     (22, 3, 172),
    ],
    [
     (3, 2, 224),
     (6, 2, 224),
     (10, 2, 224),
     (15, 2, 224),
     (24, 2, 224),
     (31, 2, 224),
     (41, 2, 224),
     (56, 3, 224),
     (3, 2, 226),
     (6, 2, 226),
     (10, 2, 226),
     (15, 2, 226),
     (24, 2, 226),
     (31, 2, 226),
     (41, 2, 226),
     (56, 3, 226),
    ],
    [
     (2, 2, 153),
     (9, 2, 153),
     (23, 2, 153),
     (40, 3, 153),
     (2, 2, 161),
     (9, 2, 161),
     (23, 2, 161),
     (40, 3, 161),
     (2, 2, 167),
     (9, 2, 167),
     (23, 2, 167),
     (40, 3, 167),
     (2, 2, 172),
     (9, 2, 172),
     (23, 2, 172),
     (40, 3, 172),
    ],
    [
     (3, 2, 153),
     (6, 2, 153),
     (10, 2, 153),
     (15, 2, 153),
     (24, 2, 153),
     (31, 2, 153),
     (41, 2, 153),
     (56, 3, 153),
     (3, 2, 161),
     (6, 2, 161),
     (10, 2, 161),
     (15, 2, 161),
     (24, 2, 161),
     (31, 2, 161),
     (41, 2, 161),
     (56, 3, 161),
    ],
    [
     (3, 2, 167),
     (6, 2, 167),
     (10, 2, 167),
     (15, 2, 167),
     (24, 2, 167),
     (31, 2, 167),
     (41, 2, 167),
     (56, 3, 167),
     (3, 2, 172),
     (6, 2, 172),
     (10, 2, 172),
     (15, 2, 172),
     (24, 2, 172),
     (31, 2, 172),
     (41, 2, 172),
     (56, 3, 172),
    ],
    [
     (114, 0, 0),
     (115, 0, 0),
     (117, 0, 0),
     (118, 0, 0),
     (121, 0, 0),
     (123, 0, 0),
     (127, 0, 0),
     (130, 0, 0),
     (136, 0, 0),
     (139, 0, 0),
     (143, 0, 0),
     (146, 0, 0),
     (155, 0, 0),
     (162, 0, 0),
     (170, 0, 0),
     (180, 0, 0),
    ],
    [
     (0, 3, 176),
     (0, 3, 177),
     (0, 3, 179),
     (0, 3, 209),
     (0, 3, 216),
     (0, 3, 217),
     (0, 3, 227),
     (0, 3, 229),
     (0, 3, 230),
     (122, 0, 0),
     (124, 0, 0),
     (125, 0, 0),
     (128, 0, 0),
     (129, 0, 0),
     (131, 0, 0),
     (132, 0, 0),
    ],
    [
     (1, 2, 176),
     (22, 3, 176),
     (1, 2, 177),
     (22, 3, 177),
     (1, 2, 179),
     (22, 3, 179),
     (1, 2, 209),
     (22, 3, 209),
     (1, 2, 216),
     (22, 3, 216),
     (1, 2, 217),
     (22, 3, 217),
     (1, 2, 227),
     (22, 3, 227),
     (1, 2, 229),
     (22, 3, 229),
    ],
    [
     (2, 2, 176),
     (9, 2, 176),
     (23, 2, 176),
     (40, 3, 176),
     (2, 2, 177),
     (9, 2, 177),
     (23, 2, 177),
     (40, 3, 177),
     (2, 2, 179),
     (9, 2, 179),
     (23, 2, 179),
     (40, 3, 179),
     (2, 2, 209),
     (9, 2, 209),
     (23, 2, 209),
     (40, 3, 209),
    ],
    [
     (3, 2, 176),
     (6, 2, 176),
     (10, 2, 176),
     (15, 2, 176),
     (24, 2, 176),
     (31, 2, 176),
     (41, 2, 176),
     (56, 3, 176),
     (3, 2, 177),
     (6, 2, 177),
     (10, 2, 177),
     (15, 2, 177),
     (24, 2, 177),
     (31, 2, 177),
     (41, 2, 177),
     (56, 3, 177),
    ],
    [
     (3, 2, 179),
     (6, 2, 179),
     (10, 2, 179),
     (15, 2, 179),
     (24, 2, 179),
     (31, 2, 179),
     (41, 2, 179),
     (56, 3, 179),
     (3, 2, 209),
     (6, 2, 209),
     (10, 2, 209),
     (15, 2, 209),
     (24, 2, 209),
     (31, 2, 209),
     (41, 2, 209),
     (56, 3, 209),
    ],
    [
     (2, 2, 216),
     (9, 2, 216),
     (23, 2, 216),
     (40, 3, 216),
     (2, 2, 217),
     (9, 2, 217),
     (23, 2, 217),
     (40, 3, 217),
     (2, 2, 227),
     (9, 2, 227),
     (23, 2, 227),
     (40, 3, 227),
     (2, 2, 229),
     (9, 2, 229),
     (23, 2, 229),
     (40, 3, 229),
    ],
    [
     (3, 2, 216),
     (6, 2, 216),
     (10, 2, 216),
     (15, 2, 216),
     (24, 2, 216),
     (31, 2, 216),
     (41, 2, 216),
     (56, 3, 216),
     (3, 2, 217),
     (6, 2, 217),
     (10, 2, 217),
     (15, 2, 217),
     (24, 2, 217),
     (31, 2, 217),
     (41, 2, 217),
     (56, 3, 217),
    ],
    [
     (3, 2, 227),
     (6, 2, 227),
     (10, 2, 227),
     (15, 2, 227),
     (24, 2, 227),
     (31, 2, 227),
     (41, 2, 227),
     (56, 3, 227),
     (3, 2, 229),
     (6, 2, 229),
     (10, 2, 229),
     (15, 2, 229),
     (24, 2, 229),
     (31, 2, 229),
     (41, 2, 229),
     (56, 3, 229),
    ],
    [
     (1, 2, 230),
     (22, 3, 230),
     (0, 3, 129),
     (0, 3, 132),
     (0, 3, 133),
     (0, 3, 134),
     (0, 3, 136),
     (0, 3, 146),
     (0, 3, 154),
     (0, 3, 156),
     (0, 3, 160),
     (0, 3, 163),
     (0, 3, 164),
     (0, 3, 169),
     (0, 3, 170),
     (0, 3, 173),
    ],
    [
     (2, 2, 230),
     (9, 2, 230),
     (23, 2, 230),
     (40, 3, 230),
     (1, 2, 129),
     (22, 3, 129),
     (1, 2, 132),
     (22, 3, 132),
     (1, 2, 133),
     (22, 3, 133),
     (1, 2, 134),
     (22, 3, 134),
     (1, 2, 136),
     (22, 3, 136),
     (1, 2, 146),
     (22, 3, 146),
    ],
    [
     (3, 2, 230),
     (6, 2, 230),
     (10, 2, 230),
     (15, 2, 230),
     (24, 2, 230),
     (31, 2, 230),
     (41, 2, 230),
     (56, 3, 230),
     (2, 2, 129),
     (9, 2, 129),
     (23, 2, 129),
     (40, 3, 129),
     (2, 2, 132),
     (9, 2, 132),
     (23, 2, 132),
     (40, 3, 132),
    ],
    [
     (3, 2, 129),
     (6, 2, 129),
     (10, 2, 129),
     (15, 2, 129),
     (24, 2, 129),
     (31, 2, 129),
     (41, 2, 129),
     (56, 3, 129),
     (3, 2, 132),
     (6, 2, 132),
     (10, 2, 132),
     (15, 2, 132),
     (24, 2, 132),
     (31, 2, 132),
     (41, 2, 132),
     (56, 3, 132),
    ],
    [
     (2, 2, 133),
     (9, 2, 133),
     (23, 2, 133),
     (40, 3, 133),
     (2, 2, 134),
     (9, 2, 134),
     (23, 2, 134),
     (40, 3, 134),
     (2, 2, 136),
     (9, 2, 136),
     (23, 2, 136),
     (40, 3, 136),
     (2, 2, 146),
     (9, 2, 146),
     (23, 2, 146),
     (40, 3, 146),
    ],
    [
     (3, 2, 133),
     (6, 2, 133),
     (10, 2, 133),
     (15, 2, 133),
     (24, 2, 133),
     (31, 2, 133),
     (41, 2, 133),
     (56, 3, 133),
     (3, 2, 134),
     (6, 2, 134),
     (10, 2, 134),
     (15, 2, 134),
     (24, 2, 134),
     (31, 2, 134),
     (41, 2, 134),
     (56, 3, 134),
    ],
    [
     (3, 2, 136),
     (6, 2, 136),
     (10, 2, 136),
     (15, 2, 136),
     (24, 2, 136),
     (31, 2, 136),
     (41, 2, 136),
     (56, 3, 136),
     (3, 2, 146),
     (6, 2, 146),
     (10, 2, 146),
     (15, 2, 146),
     (24, 2, 146),
     (31, 2, 146),
     (41, 2, 146),
     (56, 3, 146),
    ],
    [
     (1, 2, 154),
     (22, 3, 154),
     (1, 2, 156),
     (22, 3, 156),
     (1, 2, 160),
     (22, 3, 160),
     (1, 2, 163),
     (22, 3, 163),
     (1, 2, 164),
     (22, 3, 164),
     (1, 2, 169),
     (22, 3, 169),
     (1, 2, 170),
     (22, 3, 170),
     (1, 2, 173),
     (22, 3, 173),
    ],
    [
     (2, 2, 154),
     (9, 2, 154),
     (23, 2, 154),
     (40, 3, 154),
     (2, 2, 156),
     (9, 2, 156),
     (23, 2, 156),
     (40, 3, 156),
     (2, 2, 160),
     (9, 2, 160),
     (23, 2, 160),
     (40, 3, 160),
     (2, 2, 163),
     (9, 2, 163),
     (23, 2, 163),
     (40, 3, 163),
    ],
    [
     (3, 2, 154),
     (6, 2, 154),
     (10, 2, 154),
     (15, 2, 154),
     (24, 2, 154),
     (31, 2, 154),
     (41, 2, 154),
     (56, 3, 154),
     (3, 2, 156),
     (6, 2, 156),
     (10, 2, 156),
     (15, 2, 156),
     (24, 2, 156),
     (31, 2, 156),
     (41, 2, 156),
     (56, 3, 156),
    ],
    [
     (3, 2, 160),
     (6, 2, 160),
     (10, 2, 160),
     (15, 2, 160),
     (24, 2, 160),
     (31, 2, 160),
     (41, 2, 160),
     (56, 3, 160),
     (3, 2, 163),
     (6, 2, 163),
     (10, 2, 163),
     (15, 2, 163),
     (24, 2, 163),
     (31, 2, 163),
     (41, 2, 163),
     (56, 3, 163),
    ],
    [
     (2, 2, 164),
     (9, 2, 164),
     (23, 2, 164),
     (40, 3, 164),
     (2, 2, 169),
     (9, 2, 169),
     (23, 2, 169),
     (40, 3, 169),
     (2, 2, 170),
     (9, 2, 170),
     (23, 2, 170),
     (40, 3, 170),
     (2, 2, 173),
     (9, 2, 173),
     (23, 2, 173),
     (40, 3, 173),
    ],
    [
     (3, 2, 164),
     (6, 2, 164),
     (10, 2, 164),
     (15, 2, 164),
     (24, 2, 164),
     (31, 2, 164),
     (41, 2, 164),
     (56, 3, 164),
     (3, 2, 169),
     (6, 2, 169),
     (10, 2, 169),
     (15, 2, 169),
     (24, 2, 169),
     (31, 2, 169),
     (41, 2, 169),
     (56, 3, 169),
    ],
    [
     (3, 2, 170),
     (6, 2, 170),
     (10, 2, 170),
     (15, 2, 170),
     (24, 2, 170),
     (31, 2, 170),
     (41, 2, 170),
     (56, 3, 170),
     (3, 2, 173),
     (6, 2, 173),
     (10, 2, 173),
     (15, 2, 173),
     (24, 2, 173),
     (31, 2, 173),
     (41, 2, 173),
     (56, 3, 173),
    ],
    [
     (137, 0, 0),
     (138, 0, 0),
     (140, 0, 0),
     (141, 0, 0),
     (144, 0, 0),
     (145, 0, 0),
     (147, 0, 0),
     (150, 0, 0),
     (156, 0, 0),
     (159, 0, 0),
     (163, 0, 0),
     (166, 0, 0),
     (171, 0, 0),
     (174, 0, 0),
     (181, 0, 0),
     (190, 0, 0),
    ],
    [
     (0, 3, 178),
     (0, 3, 181),
     (0, 3, 185),
     (0, 3, 186),
     (0, 3, 187),
     (0, 3, 189),
     (0, 3, 190),
     (0, 3, 196),
     (0, 3, 198),
     (0, 3, 228),
     (0, 3, 232),
     (0, 3, 233),
     (148, 0, 0),
     (149, 0, 0),
     (151, 0, 0),
     (152, 0, 0),
    ],
    [
     (1, 2, 178),
     (22, 3, 178),
     (1, 2, 181),
     (22, 3, 181),
     (1, 2, 185),
     (22, 3, 185),
     (1, 2, 186),
     (22, 3, 186),
     (1, 2, 187),
     (22, 3, 187),
     (1, 2, 189),
     (22, 3, 189),
     (1, 2, 190),
     (22, 3, 190),
     (1, 2, 196),
     (22, 3, 196),
    ],
    [
     (2, 2, 178),
     (9, 2, 178),
     (23, 2, 178),
     (40, 3, 178),
     (2, 2, 181),
     (9, 2, 181),
     (23, 2, 181),
     (40, 3, 181),
     (2, 2, 185),
     (9, 2, 185),
     (23, 2, 185),
     (40, 3, 185),
     (2, 2, 186),
     (9, 2, 186),
     (23, 2, 186),
     (40, 3, 186),
    ],
    [
     (3, 2, 178),
     (6, 2, 178),
     (10, 2, 178),
     (15, 2, 178),
     (24, 2, 178),
     (31, 2, 178),
     (41, 2, 178),
     (56, 3, 178),
     (3, 2, 181),
     (6, 2, 181),
     (10, 2, 181),
     (15, 2, 181),
     (24, 2, 181),
     (31, 2, 181),
     (41, 2, 181),
     (56, 3, 181),
    ],
    [
     (3, 2, 185),
     (6, 2, 185),
     (10, 2, 185),
     (15, 2, 185),
     (24, 2, 185),
     (31, 2, 185),
     (41, 2, 185),
     (56, 3, 185),
     (3, 2, 186),
     (6, 2, 186),
     (10, 2, 186),
     (15, 2, 186),
     (24, 2, 186),
     (31, 2, 186),
     (41, 2, 186),
     (56, 3, 186),
    ],
    [
     (2, 2, 187),
     (9, 2, 187),
     (23, 2, 187),
     (40, 3, 187),
     (2, 2, 189),
     (9, 2, 189),
     (23, 2, 189),
     (40, 3, 189),
     (2, 2, 190),
     (9, 2, 190),
     (23, 2, 190),
     (40, 3, 190),
     (2, 2, 196),
     (9, 2, 196),
     (23, 2, 196),
     (40, 3, 196),
    ],
    [
     (3, 2, 187),
     (6, 2, 187),
     (10, 2, 187),
     (15, 2, 187),
     (24, 2, 187),
     (31, 2, 187),
     (41, 2, 187),
     (56, 3, 187),
     (3, 2, 189),
     (6, 2, 189),
     (10, 2, 189),
     (15, 2, 189),
     (24, 2, 189),
     (31, 2, 189),
     (41, 2, 189),
     (56, 3, 189),
    ],
    [
     (3, 2, 190),
     (6, 2, 190),
     (10, 2, 190),
     (15, 2, 190),
     (24, 2, 190),
     (31, 2, 190),
     (41, 2, 190),
     (56, 3, 190),
     (3, 2, 196),
     (6, 2, 196),
     (10, 2, 196),
     (15, 2, 196),
     (24, 2, 196),
     (31, 2, 196),
     (41, 2, 196),
     (56, 3, 196),
    ],
    [
     (1, 2, 198),
     (22, 3, 198),
     (1, 2, 228),
     (22, 3, 228),
     (1, 2, 232),
     (22, 3, 232),
     (1, 2, 233),
     (22, 3, 233),
     (0, 3, 1),
     (0, 3, 135),
     (0, 3, 137),
     (0, 3, 138),
     (0, 3, 139),
     (0, 3, 140),
     (0, 3, 141),
     (0, 3, 143),
    ],
    [
     (2, 2, 198),
     (9, 2, 198),
     (23, 2, 198),
     (40, 3, 198),
     (2, 2, 228),
     (9, 2, 228),
     (23, 2, 228),
     (40, 3, 228),
     (2, 2, 232),
     (9, 2, 232),
     (23, 2, 232),
     (40, 3, 232),
     (2, 2, 233),
     (9, 2, 233),
     (23, 2, 233),
     (40, 3, 233),
    ],
    [
     (3, 2, 198),
     (6, 2, 198),
     (10, 2, 198),
     (15, 2, 198),
     (24, 2, 198),
     (31, 2, 198),
     (41, 2, 198),
     (56, 3, 198),
     (3, 2, 228),
     (6, 2, 228),
     (10, 2, 228),
     (15, 2, 228),
     (24, 2, 228),
     (31, 2, 228),
     (41, 2, 228),
     (56, 3, 228),
    ],
    [
     (3, 2, 232),
     (6, 2, 232),
     (10, 2, 232),
     (15, 2, 232),
     (24, 2, 232),
     (31, 2, 232),
     (41, 2, 232),
     (56, 3, 232),
     (3, 2, 233),
     (6, 2, 233),
     (10, 2, 233),
     (15, 2, 233),
     (24, 2, 233),
     (31, 2, 233),
     (41, 2, 233),
     (56, 3, 233),
    ],
    [
     (1, 2, 1),
     (22, 3, 1),
     (1, 2, 135),
     (22, 3, 135),
     (1, 2, 137),
     (22, 3, 137),
     (1, 2, 138),
     (22, 3, 138),
     (1, 2, 139),
     (22, 3, 139),
     (1, 2, 140),
     (22, 3, 140),
     (1, 2, 141),
     (22, 3, 141),
     (1, 2, 143),
     (22, 3, 143),
    ],
    [
     (2, 2, 1),
     (9, 2, 1),
     (23, 2, 1),
     (40, 3, 1),
     (2, 2, 135),
     (9, 2, 135),
     (23, 2, 135),
     (40, 3, 135),
     (2, 2, 137),
     (9, 2, 137),
     (23, 2, 137),
     (40, 3, 137),
     (2, 2, 138),
     (9, 2, 138),
     (23, 2, 138),
     (40, 3, 138),
    ],
    [
     (3, 2, 1),
     (6, 2, 1),
     (10, 2, 1),
     (15, 2, 1),
     (24, 2, 1),
     (31, 2, 1),
     (41, 2, 1),
     (56, 3, 1),
     (3, 2, 135),
     (6, 2, 135),
     (10, 2, 135),
     (15, 2, 135),
     (24, 2, 135),
     (31, 2, 135),
     (41, 2, 135),
     (56, 3, 135),
    ],
    [
     (3, 2, 137),
     (6, 2, 137),
     (10, 2, 137),
     (15, 2, 137),
     (24, 2, 137),
     (31, 2, 137),
     (41, 2, 137),
     (56, 3, 137),
     (3, 2, 138),
     (6, 2, 138),
     (10, 2, 138),
     (15, 2, 138),
     (24, 2, 138),
     (31, 2, 138),
     (41, 2, 138),
     (56, 3, 138),
    ],
    [
     (2, 2, 139),
     (9, 2, 139),
     (23, 2, 139),
     (40, 3, 139),
     (2, 2, 140),
     (9, 2, 140),
     (23, 2, 140),
     (40, 3, 140),
     (2, 2, 141),
     (9, 2, 141),
     (23, 2, 141),
     (40, 3, 141),
     (2, 2, 143),
     (9, 2, 143),
     (23, 2, 143),
     (40, 3, 143),
    ],
    [
     (3, 2, 139),
     (6, 2, 139),
     (10, 2, 139),
     (15, 2, 139),
     (24, 2, 139),
     (31, 2, 139),
     (41, 2, 139),
     (56, 3, 139),
     (3, 2, 140),
     (6, 2, 140),
     (10, 2, 140),
     (15, 2, 140),
     (24, 2, 140),
     (31, 2, 140),
     (41, 2, 140),
     (56, 3, 140),
    ],
    [
     (3, 2, 141),
     (6, 2, 141),
     (10, 2, 141),
     (15, 2, 141),
     (24, 2, 141),
     (31, 2, 141),
     (41, 2, 141),
     (56, 3, 141),
     (3, 2, 143),
     (6, 2, 143),
     (10, 2, 143),
     (15, 2, 143),
     (24, 2, 143),
     (31, 2, 143),
     (41, 2, 143),
     (56, 3, 143),
    ],
    [
     (157, 0, 0),
     (158, 0, 0),
     (160, 0, 0),
     (161, 0, 0),
     (164, 0, 0),
     (165, 0, 0),
     (167, 0, 0),
     (168, 0, 0),
     (172, 0, 0),
     (173, 0, 0),
     (175, 0, 0),
     (177, 0, 0),
     (182, 0, 0),
     (185, 0, 0),
     (191, 0, 0),
     (207, 0, 0),
    ],
    [
     (0, 3, 147),
     (0, 3, 149),
     (0, 3, 150),
     (0, 3, 151),
     (0, 3, 152),
     (0, 3, 155),
     (0, 3, 157),
     (0, 3, 158),
     (0, 3, 165),
     (0, 3, 166),
     (0, 3, 168),
     (0, 3, 174),
     (0, 3, 175),
     (0, 3, 180),
     (0, 3, 182),
     (0, 3, 183),
    ],
    [
     (1, 2, 147),
     (22, 3, 147),
     (1, 2, 149),
     (22, 3, 149),
     (1, 2, 150),
     (22, 3, 150),
     (1, 2, 151),
     (22, 3, 151),
     (1, 2, 152),
     (22, 3, 152),
     (1, 2, 155),
     (22, 3, 155),
     (1, 2, 157),
     (22, 3, 157),
     (1, 2, 158),
     (22, 3, 158),
    ],
    [
     (2, 2, 147),
     (9, 2, 147),
     (23, 2, 147),
     (40, 3, 147),
     (2, 2, 149),
     (9, 2, 149),
     (23, 2, 149),
     (40, 3, 149),
     (2, 2, 150),
     (9, 2, 150),
     (23, 2, 150),
     (40, 3, 150),
     (2, 2, 151),
     (9, 2, 151),
     (23, 2, 151),
     (40, 3, 151),
    ],
    [
     (3, 2, 147),
     (6, 2, 147),
     (10, 2, 147),
     (15, 2, 147),
     (24, 2, 147),
     (31, 2, 147),
     (41, 2, 147),
     (56, 3, 147),
     (3, 2, 149),
     (6, 2, 149),
     (10, 2, 149),
     (15, 2, 149),
     (24, 2, 149),
     (31, 2, 149),
     (41, 2, 149),
     (56, 3, 149),
    ],
    [
     (3, 2, 150),
     (6, 2, 150),
     (10, 2, 150),
     (15, 2, 150),
     (24, 2, 150),
     (31, 2, 150),
     (41, 2, 150),
     (56, 3, 150),
     (3, 2, 151),
     (6, 2, 151),
     (10, 2, 151),
     (15, 2, 151),
     (24, 2, 151),
     (31, 2, 151),
     (41, 2, 151),
     (56, 3, 151),
    ],
    [
     (2, 2, 152),
     (9, 2, 152),
     (23, 2, 152),
     (40, 3, 152),
     (2, 2, 155),
     (9, 2, 155),
     (23, 2, 155),
     (40, 3, 155),
     (2, 2, 157),
     (9, 2, 157),
     (23, 2, 157),
     (40, 3, 157),
     (2, 2, 158),
     (9, 2, 158),
     (23, 2, 158),
     (40, 3, 158),
    ],
    [
     (3, 2, 152),
     (6, 2, 152),
     (10, 2, 152),
     (15, 2, 152),
     (24, 2, 152),
     (31, 2, 152),
     (41, 2, 152),
     (56, 3, 152),
     (3, 2, 155),
     (6, 2, 155),
     (10, 2, 155),
     (15, 2, 155),
     (24, 2, 155),
     (31, 2, 155),
     (41, 2, 155),
     (56, 3, 155),
    ],
    [
     (3, 2, 157),
     (6, 2, 157),
     (10, 2, 157),
     (15, 2, 157),
     (24, 2, 157),
     (31, 2, 157),
     (41, 2, 157),
     (56, 3, 157),
     (3, 2, 158),
     (6, 2, 158),
     (10, 2, 158),
     (15, 2, 158),
     (24, 2, 158),
     (31, 2, 158),
     (41, 2, 158),
     (56, 3, 158),
    ],
    [
     (1, 2, 165),
     (22, 3, 165),
     (1, 2, 166),
     (22, 3, 166),
     (1, 2, 168),
     (22, 3, 168),
     (1, 2, 174),
     (22, 3, 174),
     (1, 2, 175),
     (22, 3, 175),
     (1, 2, 180),
     (22, 3, 180),
     (1, 2, 182),
     (22, 3, 182),
     (1, 2, 183),
     (22, 3, 183),
    ],
    [
     (2, 2, 165),
     (9, 2, 165),
     (23, 2, 165),
     (40, 3, 165),
     (2, 2, 166),
     (9, 2, 166),
     (23, 2, 166),
     (40, 3, 166),
     (2, 2, 168),
     (9, 2, 168),
     (23, 2, 168),
     (40, 3, 168),
     (2, 2, 174),
     (9, 2, 174),
     (23, 2, 174),
     (40, 3, 174),
    ],
    [
     (3, 2, 165),
     (6, 2, 165),
     (10, 2, 165),
     (15, 2, 165),
     (24, 2, 165),
     (31, 2, 165),
     (41, 2, 165),
     (56, 3, 165),
     (3, 2, 166),
     (6, 2, 166),
     (10, 2, 166),
     (15, 2, 166),
     (24, 2, 166),
     (31, 2, 166),
     (41, 2, 166),
     (56, 3, 166),
    ],
    [
     (3, 2, 168),
     (6, 2, 168),
     (10, 2, 168),
     (15, 2, 168),
     (24, 2, 168),
     (31, 2, 168),
     (41, 2, 168),
     (56, 3, 168),
     (3, 2, 174),
     (6, 2, 174),
     (10, 2, 174),
     (15, 2, 174),
     (24, 2, 174),
     (31, 2, 174),
     (41, 2, 174),
     (56, 3, 174),
    ],
    [
     (2, 2, 175),
     (9, 2, 175),
     (23, 2, 175),
     (40, 3, 175),
     (2, 2, 180),
     (9, 2, 180),
     (23, 2, 180),
     (40, 3, 180),
     (2, 2, 182),
     (9, 2, 182),
     (23, 2, 182),
     (40, 3, 182),
     (2, 2, 183),
     (9, 2, 183),
     (23, 2, 183),
     (40, 3, 183),
    ],
    [
     (3, 2, 175),
     (6, 2, 175),
     (10, 2, 175),
     (15, 2, 175),
     (24, 2, 175),
     (31, 2, 175),
     (41, 2, 175),
     (56, 3, 175),
     (3, 2, 180),
     (6, 2, 180),
     (10, 2, 180),
     (15, 2, 180),
     (24, 2, 180),
     (31, 2, 180),
     (41, 2, 180),
     (56, 3, 180),
    ],
    [
     (3, 2, 182),
     (6, 2, 182),
     (10, 2, 182),
     (15, 2, 182),
     (24, 2, 182),
     (31, 2, 182),
     (41, 2, 182),
     (56, 3, 182),
     (3, 2, 183),
     (6, 2, 183),
     (10, 2, 183),
     (15, 2, 183),
     (24, 2, 183),
     (31, 2, 183),
     (41, 2, 183),
     (56, 3, 183),
    ],
    [
     (0, 3, 188),
     (0, 3, 191),
     (0, 3, 197),
     (0, 3, 231),
     (0, 3, 239),
     (176, 0, 0),
     (178, 0, 0),
     (179, 0, 0),
     (183, 0, 0),
     (184, 0, 0),
     (186, 0, 0),
     (187, 0, 0),
     (192, 0, 0),
     (199, 0, 0),
     (208, 0, 0),
     (223, 0, 0),
    ],
    [
     (1, 2, 188),
     (22, 3, 188),
     (1, 2, 191),
     (22, 3, 191),
     (1, 2, 197),
     (22, 3, 197),
     (1, 2, 231),
     (22, 3, 231),
     (1, 2, 239),
     (22, 3, 239),
     (0, 3, 9),
     (0, 3, 142),
     (0, 3, 144),
     (0, 3, 145),
     (0, 3, 148),
     (0, 3, 159),
    ],
    [
     (2, 2, 188),
     (9, 2, 188),
     (23, 2, 188),
     (40, 3, 188),
     (2, 2, 191),
     (9, 2, 191),
     (23, 2, 191),
     (40, 3, 191),
     (2, 2, 197),
     (9, 2, 197),
     (23, 2, 197),
     (40, 3, 197),
     (2, 2, 231),
     (9, 2, 231),
     (23, 2, 231),
     (40, 3, 231),
    ],
    [
     (3, 2, 188),
     (6, 2, 188),
     (10, 2, 188),
     (15, 2, 188),
     (24, 2, 188),
     (31, 2, 188),
     (41, 2, 188),
     (56, 3, 188),
     (3, 2, 191),
     (6, 2, 191),
     (10, 2, 191),
     (15, 2, 191),
     (24, 2, 191),
     (31, 2, 191),
     (41, 2, 191),
     (56, 3, 191),
    ],
    [
     (3, 2, 197),
     (6, 2, 197),
     (10, 2, 197),
     (15, 2, 197),
     (24, 2, 197),
     (31, 2, 197),
     (41, 2, 197),
     (56, 3, 197),
     (3, 2, 231),
     (6, 2, 231),
     (10, 2, 231),
     (15, 2, 231),
     (24, 2, 231),
     (31, 2, 231),
     (41, 2, 231),
     (56, 3, 231),
    ],
    [
     (2, 2, 239),
     (9, 2, 239),
     (23, 2, 239),
     (40, 3, 239),
     (1, 2, 9),
     (22, 3, 9),
     (1, 2, 142),
     (22, 3, 142),
     (1, 2, 144),
     (22, 3, 144),
     (1, 2, 145),
     (22, 3, 145),
     (1, 2, 148),
     (22, 3, 148),
     (1, 2, 159),
     (22, 3, 159),
    ],
    [
     (3, 2, 239),
     (6, 2, 239),
     (10, 2, 239),
     (15, 2, 239),
     (24, 2, 239),
     (31, 2, 239),
     (41, 2, 239),
     (56, 3, 239),
     (2, 2, 9),
     (9, 2, 9),
     (23, 2, 9),
     (40, 3, 9),
     (2, 2, 142),
     (9, 2, 142),
     (23, 2, 142),
     (40, 3, 142),
    ],
    [
     (3, 2, 9),
     (6, 2, 9),
     (10, 2, 9),
     (15, 2, 9),
     (24, 2, 9),
     (31, 2, 9),
     (41, 2, 9),
     (56, 3, 9),
     (3, 2, 142),
     (6, 2, 142),
     (10, 2, 142),
     (15, 2, 142),
     (24, 2, 142),
     (31, 2, 142),
     (41, 2, 142),
     (56, 3, 142),
    ],
    [
     (2, 2, 144),
     (9, 2, 144),
     (23, 2, 144),
     (40, 3, 144),
     (2, 2, 145),
     (9, 2, 145),
     (23, 2, 145),
     (40, 3, 145),
     (2, 2, 148),
     (9, 2, 148),
     (23, 2, 148),
     (40, 3, 148),
     (2, 2, 159),
     (9, 2, 159),
     (23, 2, 159),
     (40, 3, 159),
    ],
    [
     (3, 2, 144),
     (6, 2, 144),
     (10, 2, 144),
     (15, 2, 144),
     (24, 2, 144),
     (31, 2, 144),
     (41, 2, 144),
     (56, 3, 144),
     (3, 2, 145),
     (6, 2, 145),
     (10, 2, 145),
     (15, 2, 145),
     (24, 2, 145),
     (31, 2, 145),
     (41, 2, 145),
     (56, 3, 145),
    ],
    [
     (3, 2, 148),
     (6, 2, 148),
     (10, 2, 148),
     (15, 2, 148),
     (24, 2, 148),
     (31, 2, 148),
     (41, 2, 148),
     (56, 3, 148),
     (3, 2, 159),
     (6, 2, 159),
     (10, 2, 159),
     (15, 2, 159),
     (24, 2, 159),
     (31, 2, 159),
     (41, 2, 159),
     (56, 3, 159),
    ],
    [
     (0, 3, 171),
     (0, 3, 206),
     (0, 3, 215),
     (0, 3, 225),
     (0, 3, 236),
     (0, 3, 237),
     (188, 0, 0),
     (189, 0, 0),
     (193, 0, 0),
     (196, 0, 0),
     (200, 0, 0),
     (203, 0, 0),
     (209, 0, 0),
     (216, 0, 0),
     (224, 0, 0),
     (238, 0, 0),
    ],
    [
     (1, 2, 171),
     (22, 3, 171),
     (1, 2, 206),
     (22, 3, 206),
     (1, 2, 215),
     (22, 3, 215),
     (1, 2, 225),
     (22, 3, 225),
     (1, 2, 236),
     (22, 3, 236),
     (1, 2, 237),
     (22, 3, 237),
     (0, 3, 199),
     (0, 3, 207),
     (0, 3, 234),
     (0, 3, 235),
    ],
    [
     (2, 2, 171),
     (9, 2, 171),
     (23, 2, 171),
     (40, 3, 171),
     (2, 2, 206),
     (9, 2, 206),
     (23, 2, 206),
     (40, 3, 206),
     (2, 2, 215),
     (9, 2, 215),
     (23, 2, 215),
     (40, 3, 215),
     (2, 2, 225),
     (9, 2, 225),
     (23, 2, 225),
     (40, 3, 225),
    ],
    [
     (3, 2, 171),
     (6, 2, 171),
     (10, 2, 171),
     (15, 2, 171),
     (24, 2, 171),
     (31, 2, 171),
     (41, 2, 171),
     (56, 3, 171),
     (3, 2, 206),
     (6, 2, 206),
     (10, 2, 206),
     (15, 2, 206),
     (24, 2, 206),
     (31, 2, 206),
     (41, 2, 206),
     (56, 3, 206),
    ],
    [
     (3, 2, 215),
     (6, 2, 215),
     (10, 2, 215),
     (15, 2, 215),
     (24, 2, 215),
     (31, 2, 215),
     (41, 2, 215),
     (56, 3, 215),
     (3, 2, 225),
     (6, 2, 225),
     (10, 2, 225),
     (15, 2, 225),
     (24, 2, 225),
     (31, 2, 225),
     (41, 2, 225),
     (56, 3, 225),
    ],
    [
     (2, 2, 236),
     (9, 2, 236),
     (23, 2, 236),
     (40, 3, 236),
     (2, 2, 237),
     (9, 2, 237),
     (23, 2, 237),
     (40, 3, 237),
     (1, 2, 199),
     (22, 3, 199),
     (1, 2, 207),
     (22, 3, 207),
     (1, 2, 234),
     (22, 3, 234),
     (1, 2, 235),
     (22, 3, 235),
    ],
    [
     (3, 2, 236),
     (6, 2, 236),
     (10, 2, 236),
     (15, 2, 236),
     (24, 2, 236),
     (31, 2, 236),
     (41, 2, 236),
     (56, 3, 236),
     (3, 2, 237),
     (6, 2, 237),
     (10, 2, 237),
     (15, 2, 237),
     (24, 2, 237),
     (31, 2, 237),
     (41, 2, 237),
     (56, 3, 237),
    ],
    [
     (2, 2, 199),
     (9, 2, 199),
     (23, 2, 199),
     (40, 3, 199),
     (2, 2, 207),
     (9, 2, 207),
     (23, 2, 207),
     (40, 3, 207),
     (2, 2, 234),
     (9, 2, 234),
     (23, 2, 234),
     (40, 3, 234),
     (2, 2, 235),
     (9, 2, 235),
     (23, 2, 235),
     (40, 3, 235),
    ],
    [
     (3, 2, 199),
     (6, 2, 199),
     (10, 2, 199),
     (15, 2, 199),
     (24, 2, 199),
     (31, 2, 199),
     (41, 2, 199),
     (56, 3, 199),
     (3, 2, 207),
     (6, 2, 207),
     (10, 2, 207),
     (15, 2, 207),
     (24, 2, 207),
     (31, 2, 207),
     (41, 2, 207),
     (56, 3, 207),
    ],
    [
     (3, 2, 234),
     (6, 2, 234),
     (10, 2, 234),
     (15, 2, 234),
     (24, 2, 234),
     (31, 2, 234),
     (41, 2, 234),
     (56, 3, 234),
     (3, 2, 235),
     (6, 2, 235),
     (10, 2, 235),
     (15, 2, 235),
     (24, 2, 235),
     (31, 2, 235),
     (41, 2, 235),
     (56, 3, 235),
    ],
    [
     (194, 0, 0),
     (195, 0, 0),
     (197, 0, 0),
     (198, 0, 0),
     (201, 0, 0),
     (202, 0, 0),
     (204, 0, 0),
     (205, 0, 0),
     (210, 0, 0),
     (213, 0, 0),
     (217, 0, 0),
     (220, 0, 0),
     (225, 0, 0),
     (231, 0, 0),
     (239, 0, 0),
     (246, 0, 0),
    ],
    [
     (0, 3, 192),
     (0, 3, 193),
     (0, 3, 200),
     (0, 3, 201),
     (0, 3, 202),
     (0, 3, 205),
     (0, 3, 210),
     (0, 3, 213),
     (0, 3, 218),
     (0, 3, 219),
     (0, 3, 238),
     (0, 3, 240),
     (0, 3, 242),
     (0, 3, 243),
     (0, 3, 255),
     (206, 0, 0),
    ],
    [
     (1, 2, 192),
     (22, 3, 192),
     (1, 2, 193),
     (22, 3, 193),
     (1, 2, 200),
     (22, 3, 200),
     (1, 2, 201),
     (22, 3, 201),
     (1, 2, 202),
     (22, 3, 202),
     (1, 2, 205),
     (22, 3, 205),
     (1, 2, 210),
     (22, 3, 210),
     (1, 2, 213),
     (22, 3, 213),
    ],
    [
     (2, 2, 192),
     (9, 2, 192),
     (23, 2, 192),
     (40, 3, 192),
     (2, 2, 193),
     (9, 2, 193),
     (23, 2, 193),
     (40, 3, 193),
     (2, 2, 200),
     (9, 2, 200),
     (23, 2, 200),
     (40, 3, 200),
     (2, 2, 201),
     (9, 2, 201),
     (23, 2, 201),
     (40, 3, 201),
    ],
    [
     (3, 2, 192),
     (6, 2, 192),
     (10, 2, 192),
     (15, 2, 192),
     (24, 2, 192),
     (31, 2, 192),
     (41, 2, 192),
     (56, 3, 192),
     (3, 2, 193),
     (6, 2, 193),
     (10, 2, 193),
     (15, 2, 193),
     (24, 2, 193),
     (31, 2, 193),
     (41, 2, 193),
     (56, 3, 193),
    ],
    [
     (3, 2, 200),
     (6, 2, 200),
     (10, 2, 200),
     (15, 2, 200),
     (24, 2, 200),
     (31, 2, 200),
     (41, 2, 200),
     (56, 3, 200),
     (3, 2, 201),
     (6, 2, 201),
     (10, 2, 201),
     (15, 2, 201),
     (24, 2, 201),
     (31, 2, 201),
     (41, 2, 201),
     (56, 3, 201),
    ],
    [
     (2, 2, 202),
     (9, 2, 202),
     (23, 2, 202),
     (40, 3, 202),
     (2, 2, 205),
     (9, 2, 205),
     (23, 2, 205),
     (40, 3, 205),
     (2, 2, 210),
     (9, 2, 210),
     (23, 2, 210),
     (40, 3, 210),
     (2, 2, 213),
     (9, 2, 213),
     (23, 2, 213),
     (40, 3, 213),
    ],
    [
     (3, 2, 202),
     (6, 2, 202),
     (10, 2, 202),
     (15, 2, 202),
     (24, 2, 202),
     (31, 2, 202),
     (41, 2, 202),
     (56, 3, 202),
     (3, 2, 205),
     (6, 2, 205),
     (10, 2, 205),
     (15, 2, 205),
     (24, 2, 205),
     (31, 2, 205),
     (41, 2, 205),
     (56, 3, 205),
    ],
    [
     (3, 2, 210),
     (6, 2, 210),
     (10, 2, 210),
     (15, 2, 210),
     (24, 2, 210),
     (31, 2, 210),
     (41, 2, 210),
     (56, 3, 210),
     (3, 2, 213),
     (6, 2, 213),
     (10, 2, 213),
     (15, 2, 213),
     (24, 2, 213),
     (31, 2, 213),
     (41, 2, 213),
     (56, 3, 213),
    ],
    [
     (1, 2, 218),
     (22, 3, 218),
     (1, 2, 219),
     (22, 3, 219),
     (1, 2, 238),
     (22, 3, 238),
     (1, 2, 240),
     (22, 3, 240),
     (1, 2, 242),
     (22, 3, 242),
     (1, 2, 243),
     (22, 3, 243),
     (1, 2, 255),
     (22, 3, 255),
     (0, 3, 203),
     (0, 3, 204),
    ],
    [
     (2, 2, 218),
     (9, 2, 218),
     (23, 2, 218),
     (40, 3, 218),
     (2, 2, 219),
     (9, 2, 219),
     (23, 2, 219),
     (40, 3, 219),
     (2, 2, 238),
     (9, 2, 238),
     (23, 2, 238),
     (40, 3, 238),
     (2, 2, 240),
     (9, 2, 240),
     (23, 2, 240),
     (40, 3, 240),
    ],
    [
     (3, 2, 218),
     (6, 2, 218),
     (10, 2, 218),
     (15, 2, 218),
     (24, 2, 218),
     (31, 2, 218),
     (41, 2, 218),
     (56, 3, 218),
     (3, 2, 219),
     (6, 2, 219),
     (10, 2, 219),
     (15, 2, 219),
     (24, 2, 219),
     (31, 2, 219),
     (41, 2, 219),
     (56, 3, 219),
    ],
    [
     (3, 2, 238),
     (6, 2, 238),
     (10, 2, 238),
     (15, 2, 238),
     (24, 2, 238),
     (31, 2, 238),
     (41, 2, 238),
     (56, 3, 238),
     (3, 2, 240),
     (6, 2, 240),
     (10, 2, 240),
     (15, 2, 240),
     (24, 2, 240),
     (31, 2, 240),
     (41, 2, 240),
     (56, 3, 240),
    ],
    [
     (2, 2, 242),
     (9, 2, 242),
     (23, 2, 242),
     (40, 3, 242),
     (2, 2, 243),
     (9, 2, 243),
     (23, 2, 243),
     (40, 3, 243),
     (2, 2, 255),
     (9, 2, 255),
     (23, 2, 255),
     (40, 3, 255),
     (1, 2, 203),
     (22, 3, 203),
     (1, 2, 204),
     (22, 3, 204),
    ],
    [
     (3, 2, 242),
     (6, 2, 242),
     (10, 2, 242),
     (15, 2, 242),
     (24, 2, 242),
     (31, 2, 242),
     (41, 2, 242),
     (56, 3, 242),
     (3, 2, 243),
     (6, 2, 243),
     (10, 2, 243),
     (15, 2, 243),
     (24, 2, 243),
     (31, 2, 243),
     (41, 2, 243),
     (56, 3, 243),
    ],
    [
     (3, 2, 255),
     (6, 2, 255),
     (10, 2, 255),
     (15, 2, 255),
     (24, 2, 255),
     (31, 2, 255),
     (41, 2, 255),
     (56, 3, 255),
     (2, 2, 203),
     (9, 2, 203),
     (23, 2, 203),
     (40, 3, 203),
     (2, 2, 204),
     (9, 2, 204),
     (23, 2, 204),
     (40, 3, 204),
    ],
    [
     (3, 2, 203),
     (6, 2, 203),
     (10, 2, 203),
     (15, 2, 203),
     (24, 2, 203),
     (31, 2, 203),
     (41, 2, 203),
     (56, 3, 203),
     (3, 2, 204),
     (6, 2, 204),
     (10, 2, 204),
     (15, 2, 204),
     (24, 2, 204),
     (31, 2, 204),
     (41, 2, 204),
     (56, 3, 204),
    ],
    [
     (211, 0, 0),
     (212, 0, 0),
     (214, 0, 0),
     (215, 0, 0),
     (218, 0, 0),
     (219, 0, 0),
     (221, 0, 0),
     (222, 0, 0),
     (226, 0, 0),
     (228, 0, 0),
     (232, 0, 0),
     (235, 0, 0),
     (240, 0, 0),
     (243, 0, 0),
     (247, 0, 0),
     (250, 0, 0),
    ],
    [
     (0, 3, 211),
     (0, 3, 212),
     (0, 3, 214),
     (0, 3, 221),
     (0, 3, 222),
     (0, 3, 223),
     (0, 3, 241),
     (0, 3, 244),
     (0, 3, 245),
     (0, 3, 246),
     (0, 3, 247),
     (0, 3, 248),
     (0, 3, 250),
     (0, 3, 251),
     (0, 3, 252),
     (0, 3, 253),
    ],
    [
     (1, 2, 211),
     (22, 3, 211),
     (1, 2, 212),
     (22, 3, 212),
     (1, 2, 214),
     (22, 3, 214),
     (1, 2, 221),
     (22, 3, 221),
     (1, 2, 222),
     (22, 3, 222),
     (1, 2, 223),
     (22, 3, 223),
     (1, 2, 241),
     (22, 3, 241),
     (1, 2, 244),
     (22, 3, 244),
    ],
    [
     (2, 2, 211),
     (9, 2, 211),
     (23, 2, 211),
     (40, 3, 211),
     (2, 2, 212),
     (9, 2, 212),
     (23, 2, 212),
     (40, 3, 212),
     (2, 2, 214),
     (9, 2, 214),
     (23, 2, 214),
     (40, 3, 214),
     (2, 2, 221),
     (9, 2, 221),
     (23, 2, 221),
     (40, 3, 221),
    ],
    [
     (3, 2, 211),
     (6, 2, 211),
     (10, 2, 211),
     (15, 2, 211),
     (24, 2, 211),
     (31, 2, 211),
     (41, 2, 211),
     (56, 3, 211),
     (3, 2, 212),
     (6, 2, 212),
     (10, 2, 212),
     (15, 2, 212),
     (24, 2, 212),
     (31, 2, 212),
     (41, 2, 212),
     (56, 3, 212),
    ],
    [
     (3, 2, 214),
     (6, 2, 214),
     (10, 2, 214),
     (15, 2, 214),
     (24, 2, 214),
     (31, 2, 214),
     (41, 2, 214),
     (56, 3, 214),
     (3, 2, 221),
     (6, 2, 221),
     (10, 2, 221),
     (15, 2, 221),
     (24, 2, 221),
     (31, 2, 221),
     (41, 2, 221),
     (56, 3, 221),
    ],
    [
     (2, 2, 222),
     (9, 2, 222),
     (23, 2, 222),
     (40, 3, 222),
     (2, 2, 223),
     (9, 2, 223),
     (23, 2, 223),
     (40, 3, 223),
     (2, 2, 241),
     (9, 2, 241),
     (23, 2, 241),
     (40, 3, 241),
     (2, 2, 244),
     (9, 2, 244),
     (23, 2, 244),
     (40, 3, 244),
    ],
    [
     (3, 2, 222),
     (6, 2, 222),
     (10, 2, 222),
     (15, 2, 222),
     (24, 2, 222),
     (31, 2, 222),
     (41, 2, 222),
     (56, 3, 222),
     (3, 2, 223),
     (6, 2, 223),
     (10, 2, 223),
     (15, 2, 223),
     (24, 2, 223),
     (31, 2, 223),
     (41, 2, 223),
     (56, 3, 223),
    ],
    [
     (3, 2, 241),
     (6, 2, 241),
     (10, 2, 241),
     (15, 2, 241),
     (24, 2, 241),
     (31, 2, 241),
     (41, 2, 241),
     (56, 3, 241),
     (3, 2, 244),
     (6, 2, 244),
     (10, 2, 244),
     (15, 2, 244),
     (24, 2, 244),
     (31, 2, 244),
     (41, 2, 244),
     (56, 3, 244),
    ],
    [
     (1, 2, 245),
     (22, 3, 245),
     (1, 2, 246),
     (22, 3, 246),
     (1, 2, 247),
     (22, 3, 247),
     (1, 2, 248),
     (22, 3, 248),
     (1, 2, 250),
     (22, 3, 250),
     (1, 2, 251),
     (22, 3, 251),
     (1, 2, 252),
     (22, 3, 252),
     (1, 2, 253),
     (22, 3, 253),
    ],
    [
     (2, 2, 245),
     (9, 2, 245),
     (23, 2, 245),
     (40, 3, 245),
     (2, 2, 246),
     (9, 2, 246),
     (23, 2, 246),
     (40, 3, 246),
     (2, 2, 247),
     (9, 2, 247),
     (23, 2, 247),
     (40, 3, 247),
     (2, 2, 248),
     (9, 2, 248),
     (23, 2, 248),
     (40, 3, 248),
    ],
    [
     (3, 2, 245),
     (6, 2, 245),
     (10, 2, 245),
     (15, 2, 245),
     (24, 2, 245),
     (31, 2, 245),
     (41, 2, 245),
     (56, 3, 245),
     (3, 2, 246),
     (6, 2, 246),
     (10, 2, 246),
     (15, 2, 246),
     (24, 2, 246),
     (31, 2, 246),
     (41, 2, 246),
     (56, 3, 246),
    ],
    [
     (3, 2, 247),
     (6, 2, 247),
     (10, 2, 247),
     (15, 2, 247),
     (24, 2, 247),
     (31, 2, 247),
     (41, 2, 247),
     (56, 3, 247),
     (3, 2, 248),
     (6, 2, 248),
     (10, 2, 248),
     (15, 2, 248),
     (24, 2, 248),
     (31, 2, 248),
     (41, 2, 248),
     (56, 3, 248),
    ],
    [
     (2, 2, 250),
     (9, 2, 250),
     (23, 2, 250),
     (40, 3, 250),
     (2, 2, 251),
     (9, 2, 251),
     (23, 2, 251),
     (40, 3, 251),
     (2, 2, 252),
     (9, 2, 252),
     (23, 2, 252),
     (40, 3, 252),
     (2, 2, 253),
     (9, 2, 253),
     (23, 2, 253),
     (40, 3, 253),
    ],
    [
     (3, 2, 250),
     (6, 2, 250),
     (10, 2, 250),
     (15, 2, 250),
     (24, 2, 250),
     (31, 2, 250),
     (41, 2, 250),
     (56, 3, 250),
     (3, 2, 251),
     (6, 2, 251),
     (10, 2, 251),
     (15, 2, 251),
     (24, 2, 251),
     (31, 2, 251),
     (41, 2, 251),
     (56, 3, 251),
    ],
    [
     (3, 2, 252),
     (6, 2, 252),
     (10, 2, 252),
     (15, 2, 252),
     (24, 2, 252),
     (31, 2, 252),
     (41, 2, 252),
     (56, 3, 252),
     (3, 2, 253),
     (6, 2, 253),
     (10, 2, 253),
     (15, 2, 253),
     (24, 2, 253),
     (31, 2, 253),
     (41, 2, 253),
     (56, 3, 253),
    ],
    [
     (0, 3, 254),
     (227, 0, 0),
     (229, 0, 0),
     (230, 0, 0),
     (233, 0, 0),
     (234, 0, 0),
     (236, 0, 0),
     (237, 0, 0),
     (241, 0, 0),
     (242, 0, 0),
     (244, 0, 0),
     (245, 0, 0),
     (248, 0, 0),
     (249, 0, 0),
     (251, 0, 0),
     (252, 0, 0),
    ],
    [
     (1, 2, 254),
     (22, 3, 254),
     (0, 3, 2),
     (0, 3, 3),
     (0, 3, 4),
     (0, 3, 5),
     (0, 3, 6),
     (0, 3, 7),
     (0, 3, 8),
     (0, 3, 11),
     (0, 3, 12),
     (0, 3, 14),
     (0, 3, 15),
     (0, 3, 16),
     (0, 3, 17),
     (0, 3, 18),
    ],
    [
     (2, 2, 254),
     (9, 2, 254),
     (23, 2, 254),
     (40, 3, 254),
     (1, 2, 2),
     (22, 3, 2),
     (1, 2, 3),
     (22, 3, 3),
     (1, 2, 4),
     (22, 3, 4),
     (1, 2, 5),
     (22, 3, 5),
     (1, 2, 6),
     (22, 3, 6),
     (1, 2, 7),
     (22, 3, 7),
    ],
    [
     (3, 2, 254),
     (6, 2, 254),
     (10, 2, 254),
     (15, 2, 254),
     (24, 2, 254),
     (31, 2, 254),
     (41, 2, 254),
     (56, 3, 254),
     (2, 2, 2),
     (9, 2, 2),
     (23, 2, 2),
     (40, 3, 2),
     (2, 2, 3),
     (9, 2, 3),
     (23, 2, 3),
     (40, 3, 3),
    ],
    [
     (3, 2, 2),
     (6, 2, 2),
     (10, 2, 2),
     (15, 2, 2),
     (24, 2, 2),
     (31, 2, 2),
     (41, 2, 2),
     (56, 3, 2),
     (3, 2, 3),
     (6, 2, 3),
     (10, 2, 3),
     (15, 2, 3),
     (24, 2, 3),
     (31, 2, 3),
     (41, 2, 3),
     (56, 3, 3),
    ],
    [
     (2, 2, 4),
     (9, 2, 4),
     (23, 2, 4),
     (40, 3, 4),
     (2, 2, 5),
     (9, 2, 5),
     (23, 2, 5),
     (40, 3, 5),
     (2, 2, 6),
     (9, 2, 6),
     (23, 2, 6),
     (40, 3, 6),
     (2, 2, 7),
     (9, 2, 7),
     (23, 2, 7),
     (40, 3, 7),
    ],
    [
     (3, 2, 4),
     (6, 2, 4),
     (10, 2, 4),
     (15, 2, 4),
     (24, 2, 4),
     (31, 2, 4),
     (41, 2, 4),
     (56, 3, 4),
     (3, 2, 5),
     (6, 2, 5),
     (10, 2, 5),
     (15, 2, 5),
     (24, 2, 5),
     (31, 2, 5),
     (41, 2, 5),
     (56, 3, 5),
    ],
    [
     (3, 2, 6),
     (6, 2, 6),
     (10, 2, 6),
     (15, 2, 6),
     (24, 2, 6),
     (31, 2, 6),
     (41, 2, 6),
     (56, 3, 6),
     (3, 2, 7),
     (6, 2, 7),
     (10, 2, 7),
     (15, 2, 7),
     (24, 2, 7),
     (31, 2, 7),
     (41, 2, 7),
     (56, 3, 7),
    ],
    [
     (1, 2, 8),
     (22, 3, 8),
     (1, 2, 11),
     (22, 3, 11),
     (1, 2, 12),
     (22, 3, 12),
     (1, 2, 14),
     (22, 3, 14),
     (1, 2, 15),
     (22, 3, 15),
     (1, 2, 16),
     (22, 3, 16),
     (1, 2, 17),
     (22, 3, 17),
     (1, 2, 18),
     (22, 3, 18),
    ],
    [
     (2, 2, 8),
     (9, 2, 8),
     (23, 2, 8),
     (40, 3, 8),
     (2, 2, 11),
     (9, 2, 11),
     (23, 2, 11),
     (40, 3, 11),
     (2, 2, 12),
     (9, 2, 12),
     (23, 2, 12),
     (40, 3, 12),
     (2, 2, 14),
     (9, 2, 14),
     (23, 2, 14),
     (40, 3, 14),
    ],
    [
     (3, 2, 8),
     (6, 2, 8),
     (10, 2, 8),
     (15, 2, 8),
     (24, 2, 8),
     (31, 2, 8),
     (41, 2, 8),
     (56, 3, 8),
     (3, 2, 11),
     (6, 2, 11),
     (10, 2, 11),
     (15, 2, 11),
     (24, 2, 11),
     (31, 2, 11),
     (41, 2, 11),
     (56, 3, 11),
    ],
    [
     (3, 2, 12),
     (6, 2, 12),
     (10, 2, 12),
     (15, 2, 12),
     (24, 2, 12),
     (31, 2, 12),
     (41, 2, 12),
     (56, 3, 12),
     (3, 2, 14),
     (6, 2, 14),
     (10, 2, 14),
     (15, 2, 14),
     (24, 2, 14),
     (31, 2, 14),
     (41, 2, 14),
     (56, 3, 14),
    ],
    [
     (2, 2, 15),
     (9, 2, 15),
     (23, 2, 15),
     (40, 3, 15),
     (2, 2, 16),
     (9, 2, 16),
     (23, 2, 16),
     (40, 3, 16),
     (2, 2, 17),
     (9, 2, 17),
     (23, 2, 17),
     (40, 3, 17),
     (2, 2, 18),
     (9, 2, 18),
     (23, 2, 18),
     (40, 3, 18),
    ],
    [
     (3, 2, 15),
     (6, 2, 15),
     (10, 2, 15),
     (15, 2, 15),
     (24, 2, 15),
     (31, 2, 15),
     (41, 2, 15),
     (56, 3, 15),
     (3, 2, 16),
     (6, 2, 16),
     (10, 2, 16),
     (15, 2, 16),
     (24, 2, 16),
     (31, 2, 16),
     (41, 2, 16),
     (56, 3, 16),
    ],
    [
     (3, 2, 17),
     (6, 2, 17),
     (10, 2, 17),
     (15, 2, 17),
     (24, 2, 17),
     (31, 2, 17),
     (41, 2, 17),
     (56, 3, 17),
     (3, 2, 18),
     (6, 2, 18),
     (10, 2, 18),
     (15, 2, 18),
     (24, 2, 18),
     (31, 2, 18),
     (41, 2, 18),
     (56, 3, 18),
    ],
    [
     (0, 3, 19),
     (0, 3, 20),
     (0, 3, 21),
     (0, 3, 23),
     (0, 3, 24),
     (0, 3, 25),
     (0, 3, 26),
     (0, 3, 27),
     (0, 3, 28),
     (0, 3, 29),
     (0, 3, 30),
     (0, 3, 31),
     (0, 3, 127),
     (0, 3, 220),
     (0, 3, 249),
     (253, 0, 0),
    ],
    [
     (1, 2, 19),
     (22, 3, 19),
     (1, 2, 20),
     (22, 3, 20),
     (1, 2, 21),
     (22, 3, 21),
     (1, 2, 23),
     (22, 3, 23),
     (1, 2, 24),
     (22, 3, 24),
     (1, 2, 25),
     (22, 3, 25),
     (1, 2, 26),
     (22, 3, 26),
     (1, 2, 27),
     (22, 3, 27),
    ],
    [
     (2, 2, 19),
     (9, 2, 19),
     (23, 2, 19),
     (40, 3, 19),
     (2, 2, 20),
     (9, 2, 20),
     (23, 2, 20),
     (40, 3, 20),
     (2, 2, 21),
     (9, 2, 21),
     (23, 2, 21),
     (40, 3, 21),
     (2, 2, 23),
     (9, 2, 23),
     (23, 2, 23),
     (40, 3, 23),
    ],
    [
     (3, 2, 19),
     (6, 2, 19),
     (10, 2, 19),
     (15, 2, 19),
     (24, 2, 19),
     (31, 2, 19),
     (41, 2, 19),
     (56, 3, 19),
     (3, 2, 20),
     (6, 2, 20),
     (10, 2, 20),
     (15, 2, 20),
     (24, 2, 20),
     (31, 2, 20),
     (41, 2, 20),
     (56, 3, 20),
    ],
    [
     (3, 2, 21),
     (6, 2, 21),
     (10, 2, 21),
     (15, 2, 21),
     (24, 2, 21),
     (31, 2, 21),
     (41, 2, 21),
     (56, 3, 21),
     (3, 2, 23),
     (6, 2, 23),
     (10, 2, 23),
     (15, 2, 23),
     (24, 2, 23),
     (31, 2, 23),
     (41, 2, 23),
     (56, 3, 23),
    ],
    [
     (2, 2, 24),
     (9, 2, 24),
     (23, 2, 24),
     (40, 3, 24),
     (2, 2, 25),
     (9, 2, 25),
     (23, 2, 25),
     (40, 3, 25),
     (2, 2, 26),
     (9, 2, 26),
     (23, 2, 26),
     (40, 3, 26),
     (2, 2, 27),
     (9, 2, 27),
     (23, 2, 27),
     (40, 3, 27),
    ],
    [
     (3, 2, 24),
     (6, 2, 24),
     (10, 2, 24),
     (15, 2, 24),
     (24, 2, 24),
     (31, 2, 24),
     (41, 2, 24),
     (56, 3, 24),
     (3, 2, 25),
     (6, 2, 25),
     (10, 2, 25),
     (15, 2, 25),
     (24, 2, 25),
     (31, 2, 25),
     (41, 2, 25),
     (56, 3, 25),
    ],
    [
     (3, 2, 26),
     (6, 2, 26),
     (10, 2, 26),
     (15, 2, 26),
     (24, 2, 26),
     (31, 2, 26),
     (41, 2, 26),
     (56, 3, 26),
     (3, 2, 27),
     (6, 2, 27),
     (10, 2, 27),
     (15, 2, 27),
     (24, 2, 27),
     (31, 2, 27),
     (41, 2, 27),
     (56, 3, 27),
    ],
    [
     (1, 2, 28),
     (22, 3, 28),
     (1, 2, 29),
     (22, 3, 29),
     (1, 2, 30),
     (22, 3, 30),
     (1, 2, 31),
     (22, 3, 31),
     (1, 2, 127),
     (22, 3, 127),
     (1, 2, 220),
     (22, 3, 220),
     (1, 2, 249),
     (22, 3, 249),
     (254, 0, 0),
     (255, 0, 0),
    ],
    [
     (2, 2, 28),
     (9, 2, 28),
     (23, 2, 28),
     (40, 3, 28),
     (2, 2, 29),
     (9, 2, 29),
     (23, 2, 29),
     (40, 3, 29),
     (2, 2, 30),
     (9, 2, 30),
     (23, 2, 30),
     (40, 3, 30),
     (2, 2, 31),
     (9, 2, 31),
     (23, 2, 31),
     (40, 3, 31),
    ],
    [
     (3, 2, 28),
     (6, 2, 28),
     (10, 2, 28),
     (15, 2, 28),
     (24, 2, 28),
     (31, 2, 28),
     (41, 2, 28),
     (56, 3, 28),
     (3, 2, 29),
     (6, 2, 29),
     (10, 2, 29),
     (15, 2, 29),
     (24, 2, 29),
     (31, 2, 29),
     (41, 2, 29),
     (56, 3, 29),
    ],
    [
     (3, 2, 30),
     (6, 2, 30),
     (10, 2, 30),
     (15, 2, 30),
     (24, 2, 30),
     (31, 2, 30),
     (41, 2, 30),
     (56, 3, 30),
     (3, 2, 31),
     (6, 2, 31),
     (10, 2, 31),
     (15, 2, 31),
     (24, 2, 31),
     (31, 2, 31),
     (41, 2, 31),
     (56, 3, 31),
    ],
    [
     (2, 2, 127),
     (9, 2, 127),
     (23, 2, 127),
     (40, 3, 127),
     (2, 2, 220),
     (9, 2, 220),
     (23, 2, 220),
     (40, 3, 220),
     (2, 2, 249),
     (9, 2, 249),
     (23, 2, 249),
     (40, 3, 249),
     (0, 3, 10),
     (0, 3, 13),
     (0, 3, 22),
     (0, 4, 0),
    ],
    [
     (3, 2, 127),
     (6, 2, 127),
     (10, 2, 127),
     (15, 2, 127),
     (24, 2, 127),
     (31, 2, 127),
     (41, 2, 127),
     (56, 3, 127),
     (3, 2, 220),
     (6, 2, 220),
     (10, 2, 220),
     (15, 2, 220),
     (24, 2, 220),
     (31, 2, 220),
     (41, 2, 220),
     (56, 3, 220),
    ],
    [
     (3, 2, 249),
     (6, 2, 249),
     (10, 2, 249),
     (15, 2, 249),
     (24, 2, 249),
     (31, 2, 249),
     (41, 2, 249),
     (56, 3, 249),
     (1, 2, 10),
     (22, 3, 10),
     (1, 2, 13),
     (22, 3, 13),
     (1, 2, 22),
     (22, 3, 22),
     (0, 4, 0),
     (0, 4, 0),
    ],
    [
     (2, 2, 10),
     (9, 2, 10),
     (23, 2, 10),
     (40, 3, 10),
     (2, 2, 13),
     (9, 2, 13),
     (23, 2, 13),
     (40, 3, 13),
     (2, 2, 22),
     (9, 2, 22),
     (23, 2, 22),
     (40, 3, 22),
     (0, 4, 0),
     (0, 4, 0),
     (0, 4, 0),
     (0, 4, 0),
    ],
    [
     (3, 2, 10),
     (6, 2, 10),
     (10, 2, 10),
     (15, 2, 10),
     (24, 2, 10),
     (31, 2, 10),
     (41, 2, 10),
     (56, 3, 10),
     (3, 2, 13),
     (6, 2, 13),
     (10, 2, 13),
     (15, 2, 13),
     (24, 2, 13),
     (31, 2, 13),
     (41, 2, 13),
     (56, 3, 13),
    ],
    [
     (3, 2, 22),
     (6, 2, 22),
     (10, 2, 22),
     (15, 2, 22),
     (24, 2, 22),
     (31, 2, 22),
     (41, 2, 22),
     (56, 3, 22),
     (0, 4, 0),
     (0, 4, 0),
     (0, 4, 0),
     (0, 4, 0),
     (0, 4, 0),
     (0, 4, 0),
     (0, 4, 0),
     (0, 4, 0),
    ],
];
